@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2022 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -75,26 +75,25 @@
 
 #define WRAPPER_SOURCE_WRAPPER      -1
 #define WRAPPER_SOURCE_PROTOCOL     -2
-#define WRAPPER_SOURCE_JVM          -3
-#define WRAPPER_SOURCE_JVM_VERSION   0  /* The value '0' is important for log_printf to print this output as a direct message (otherwise any '%' in the output will cause memory issues) */
+#define WRAPPER_SOURCE_JVM_QRY       0  /* The value '0' is important for log_printf to print this output as a direct message (otherwise any '%' in the output will cause memory issues) */
 
 /* * * Log thread constants * * */
 /* These are indexes in an array so they must be sequential, start
  *  with zero and be one less than the final WRAPPER_THREAD_COUNT */
-#define WRAPPER_THREAD_CURRENT  -1
-#define WRAPPER_THREAD_SIGNAL   0
-#define WRAPPER_THREAD_MAIN     1
-#define WRAPPER_THREAD_SRVMAIN  2
-#define WRAPPER_THREAD_TIMER    3
+#define WRAPPER_THREAD_CURRENT                  -1
+#define WRAPPER_THREAD_SIGNAL                    0
+#define WRAPPER_THREAD_MAIN                      1
+#define WRAPPER_THREAD_SRVMAIN                   2
+#define WRAPPER_THREAD_TIMER                     3
 #ifdef WIN32
- #define WRAPPER_THREAD_MESSAGE  4
- #define WRAPPER_THREAD_JAVAIO   5
+ #define WRAPPER_THREAD_MESSAGE                  4
+ #define WRAPPER_THREAD_JAVAIO                   5
 #else
- #define WRAPPER_THREAD_JAVAIN   4
- #define WRAPPER_THREAD_JAVAIO   5
+ #define WRAPPER_THREAD_JAVAIN                   4
+ #define WRAPPER_THREAD_JAVAIO                   5
 #endif
-#define WRAPPER_THREAD_STARTUP  (WRAPPER_THREAD_JAVAIO+1)
-#define WRAPPER_THREAD_COUNT    (WRAPPER_THREAD_STARTUP+1)
+#define WRAPPER_THREAD_STARTUP                   (WRAPPER_THREAD_JAVAIO+1)
+#define WRAPPER_THREAD_COUNT                     (WRAPPER_THREAD_STARTUP+1)
 
 #define MAX_LOG_SIZE 4096
 
@@ -145,7 +144,7 @@
 
 /* This can be called from within logging code that would otherwise get stuck in recursion.
  *  Log to the console exactly when it happens and then also try to get it into the log
- *  file at the next oportunity. */
+ *  file at the next opportunity. */
 extern void outOfMemoryQueued(const TCHAR *context, int id);
 
 extern void outOfMemory(const TCHAR *context, int id);
@@ -314,6 +313,8 @@ extern int syslogMessageFileRegistered(int silent);
 extern int registerSyslogMessageFile(int forceInstall, int silent);
 extern int unregisterSyslogMessageFile(int silent);
 
+extern void setJvmQrySource(const TCHAR* source);
+
 extern void resetDuration();
 
 extern int getLowLogLevel();
