@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2018 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -21,9 +21,6 @@
  #include <glob.h>
  #include <string.h>
  #include <limits.h>
- #if defined(IRIX)
-  #define PATH_MAX FILENAME_MAX
- #endif
 #endif
 
 #ifdef WIN32
@@ -37,7 +34,7 @@
 #define FILES_CHUNK 5
 
 #ifndef TRUE
-#define TRUE -1
+#define TRUE 1
 #endif
 
 #ifndef FALSE
@@ -45,7 +42,7 @@
 #endif
 
 /**
- * Returns a valid sort mode given a name: "TIMES", "NAMES_ASC", "NAMES_DEC".
+ * Returns a valid sort mode given a name: "TIMES", "NAMES_ASC", "NAMES_DEC", "NAMES_SMART".
  *  In the event of an invalid value, TIMES will be returned.
  */
 int loggerFileGetSortMode(const TCHAR *modeName) {
@@ -53,6 +50,8 @@ int loggerFileGetSortMode(const TCHAR *modeName) {
         return LOGGER_FILE_SORT_MODE_NAMES_ASC;
     } else if (strcmpIgnoreCase(modeName, TEXT("NAMES_DEC")) == 0) {
         return LOGGER_FILE_SORT_MODE_NAMES_DEC;
+    } else if (strcmpIgnoreCase(modeName, TEXT("NAMES_SMART")) == 0) {
+        return LOGGER_FILE_SORT_MODE_NAMES_SMART;
     } else {
         return LOGGER_FILE_SORT_MODE_TIMES;
     }
@@ -194,6 +193,42 @@ int compareFileNames(const TCHAR *file1, const TCHAR *file2) {
     }
 }
 
+int compareFileNamesIndex(const TCHAR *file1, const TCHAR *file2, int startIndex, int stopIndex, int startCountFromEnd, int stopCountFromEnd) {
+    TCHAR *file1_;
+    TCHAR *file2_;
+    int start;
+    int len;
+    int result;
+    
+    start = (startCountFromEnd ? (int)_tcslen(file1) - startIndex : startIndex);
+    len   = (stopCountFromEnd  ? (int)_tcslen(file1) - stopIndex  : stopIndex) - start;
+    file1_ = malloc(sizeof(TCHAR) * (len + 1));
+    if (!file1_) {
+        outOfMemoryQueued(TEXT("CFNI"), 1);
+        return 0;
+    }
+    _tcsncpy(file1_, file1 + start, len);
+    file1_[len] = 0;
+    
+    start = (startCountFromEnd ? (int)_tcslen(file2) - startIndex : startIndex);
+    len   = (stopCountFromEnd  ? (int)_tcslen(file2) - stopIndex  : stopIndex) - start;
+    file2_ = malloc(sizeof(TCHAR) * (len + 1));
+    if (!file2_) {
+        free(file1_);
+        outOfMemoryQueued(TEXT("CFNI"), 2);
+        return 0;
+    }
+    _tcsncpy(file2_, file2 + start, len);
+    file2_[len] = 0;
+    
+    result = compareFileNames(file1_, file2_);
+    
+    free(file1_);
+    free(file2_);
+    
+    return result;
+}
+
 int sortFilesNamesAsc(TCHAR **files, int cnt) {
     int i, j;
     TCHAR *temp;
@@ -232,6 +267,94 @@ int sortFilesNamesDec(TCHAR **files, int cnt) {
     return TRUE;
 }
 
+int sortFilesNamesDecIndex(TCHAR **files, int cnt, int startIndex, int stopIndex, int startCountFromEnd, int stopCountFromEnd) {
+    int i, j;
+    TCHAR *temp;
+    int cmp;
+
+    for (i = 0; i < cnt; i++) {
+        for (j = 0; j < cnt - 1; j++) {
+            cmp = compareFileNamesIndex(files[j], files[j+1], startIndex, stopIndex, startCountFromEnd, stopCountFromEnd);
+            if (cmp > 0) {
+                temp = files[j + 1];
+                files[j + 1] = files[j];
+                files[j] = temp;
+            }
+        }
+    }
+
+    return TRUE;
+}
+
+/**
+ * This function allows to sort filenames with the following logic:
+ *  - if the given pattern contains a ???????? (date) token, the files are first sorted by date descending.
+ *  - if the given pattern contains a * (num) token, the files are sorted by ascending numbers (for equivalent dates).
+ *     The first file being the one without number.
+ *  NOTE: The function assumes that there is at most one token of each!
+ *
+ * @param pattern pattern used to figure out how to sort the files.
+ * @param files list of files to sort.
+ * @param cnt number of files to sort.
+ */
+int sortFilesNamesSmart(const TCHAR* pattern, TCHAR **files, int cnt) {
+    int i, j;
+    TCHAR *temp;
+    int cmp;
+    TCHAR* numToken;
+    TCHAR* dateToken;
+    int numStartIndex, numStopIndex;
+    int dateStartIndex = 0;
+    int dateStopIndex = 0;
+    int startCountFromEnd = 0;
+    int stopCountFromEnd = 0;
+    
+    /* First sort by date. */
+    dateToken = _tcsstr(pattern, TEXT("?"));
+    numToken = _tcsstr(pattern, TEXT("*"));
+    
+    if (dateToken) {
+        if (!numToken || (dateToken < numToken)) {
+            dateStartIndex = (int)(dateToken - pattern);
+            dateStopIndex = dateStartIndex + 8;
+            startCountFromEnd = FALSE;
+            stopCountFromEnd = FALSE;
+        } else {
+            /* There is a num token before the date. So the length before the date is not fixed. Calculate the index from the end. */
+            dateStartIndex = (int)_tcslen(pattern) - (int)(dateToken - pattern);
+            dateStopIndex = dateStartIndex - 8;
+            startCountFromEnd = TRUE;
+            stopCountFromEnd = TRUE;
+        }
+        sortFilesNamesDecIndex(files, cnt, dateStartIndex, dateStopIndex, startCountFromEnd, stopCountFromEnd);
+    }
+    
+    if (numToken) {
+        numStartIndex = (int)(numToken - pattern);
+        numStopIndex = (int)_tcslen(pattern) - (numStartIndex + 1);
+        for (i = 0; i < cnt; i++) {
+            for (j = 0; j < cnt - 1; j++) {
+                if (dateToken) {
+                    /* Make sure that the dates are equals. */
+                    cmp = compareFileNamesIndex(files[j], files[j+1], dateStartIndex, dateStopIndex, startCountFromEnd, stopCountFromEnd);
+                    if (cmp != 0) {
+                        continue;
+                    }
+                }
+                /* Sort by ascending name. */
+                cmp = compareFileNamesIndex(files[j], files[j+1], numStartIndex, numStopIndex, FALSE, TRUE);
+                if (cmp < 0) {
+                    temp = files[j + 1];
+                    files[j + 1] = files[j];
+                    files[j] = temp;
+                }
+            }
+        }
+    }
+
+    return TRUE;
+}
+
 /**
  * Returns a NULL terminated list of file names within the specified pattern.
  *  The files will be sorted new to old for TIMES.  Then incremental ordering
@@ -254,9 +377,9 @@ TCHAR** loggerFileGetFiles(const TCHAR* pattern, int sortMode) {
     __time64_t *fileTimes;
     __time64_t *newFileTimes;
 #else
-#ifdef WRAPPER_FILE_DEBUG
+ #ifdef WRAPPER_FILE_DEBUG
     int i;
-#endif
+ #endif
     int result;
     glob_t g;
     int findex;
@@ -294,6 +417,8 @@ TCHAR** loggerFileGetFiles(const TCHAR* pattern, int sortMode) {
         dirPart = malloc(sizeof(TCHAR) * 1);
         if (!dirPart) {
             outOfMemoryQueued(TEXT("WFGF"), 3);
+            free(files);
+            free(fileTimes);
             return NULL;
         }
         dirPart[0] = TEXT('\0');
@@ -304,23 +429,25 @@ TCHAR** loggerFileGetFiles(const TCHAR* pattern, int sortMode) {
         dirPart = malloc(sizeof(TCHAR) * (dirLen + 1));
         if (!dirPart) {
             outOfMemoryQueued(TEXT("WFGF"), 4);
+            free(files);
+            free(fileTimes);
             return NULL;
         }
         _tcsncpy(dirPart, pattern, dirLen);
         dirPart[dirLen] = TEXT('\0');
     }
 
-#ifdef WRAPPER_FILE_DEBUG
+ #ifdef WRAPPER_FILE_DEBUG
     _tprintf(TEXT("  dirPart=[%s]\n"), dirPart);
-#endif
+ #endif
 
     /* Get the first file. */
-#ifdef _IA64_
+ #ifdef _IA64_
     /* On Itanium, the first parameter is not a "const". If you don't cast it, then you have a warning */
     if ((handle = _tfindfirst64((TCHAR *)pattern, &fblock)) > 0) {
-#else
+ #else
     if ((handle = _tfindfirst64(pattern, &fblock)) > 0) {
-#endif
+ #endif
         if ((_tcscmp(fblock.name, TEXT(".")) != 0) && (_tcscmp(fblock.name, TEXT("..")) != 0)) {
             fileLen = _tcslen(fblock.name);
             files[cnt] = malloc((_tcslen(dirPart) + _tcslen(fblock.name) + 1) * sizeof(TCHAR));
@@ -333,9 +460,9 @@ TCHAR** loggerFileGetFiles(const TCHAR* pattern, int sortMode) {
             }
             _sntprintf(files[cnt], _tcslen(dirPart) + _tcslen(fblock.name) + 1, TEXT("%s%s"), dirPart, fblock.name);
             fileTimes[cnt] = fblock.time_write;
-#ifdef WRAPPER_FILE_DEBUG
+ #ifdef WRAPPER_FILE_DEBUG
             _tprintf(TEXT("  files[%d]=%s, %ld\n"), cnt, files[cnt], fileTimes[cnt]);
-#endif
+ #endif
 
             cnt++;
         }
@@ -374,9 +501,9 @@ TCHAR** loggerFileGetFiles(const TCHAR* pattern, int sortMode) {
                     files = newFiles;
                     fileTimes = newFileTimes;
                     filesSize += FILES_CHUNK;
-#ifdef WRAPPER_FILE_DEBUG
+ #ifdef WRAPPER_FILE_DEBUG
                     _tprintf(TEXT("  increased files to %d\n"), filesSize);
-#endif
+ #endif
                 }
 
                 fileLen = _tcslen(fblock.name);
@@ -391,9 +518,9 @@ TCHAR** loggerFileGetFiles(const TCHAR* pattern, int sortMode) {
                 _sntprintf(files[cnt], _tcslen(dirPart) + _tcslen(fblock.name) + 1, TEXT("%s%s"), dirPart, fblock.name);
                 fileTimes[cnt] = fblock.time_write;
 
-#ifdef WRAPPER_FILE_DEBUG
+ #ifdef WRAPPER_FILE_DEBUG
                 _tprintf(TEXT("  files[%d]=%s, %ld\n"), cnt, files[cnt], fileTimes[cnt]);
-#endif
+ #endif
                 cnt++;
             }
         }
@@ -401,13 +528,14 @@ TCHAR** loggerFileGetFiles(const TCHAR* pattern, int sortMode) {
         /* Close the file search */
         _findclose(handle);
     }
+    free(dirPart);
 
     if (cnt <= 0) {
         if (errno == ENOENT) {
             /* No files matched. */
-#ifdef WRAPPER_FILE_DEBUG
+ #ifdef WRAPPER_FILE_DEBUG
             _tprintf(TEXT("  No files matched.\n"));
-#endif
+ #endif
         } else {
             /* Encountered an error of some kind. */
             log_printf_queue(TRUE, WRAPPER_SOURCE_WRAPPER, LEVEL_ERROR, TEXT("Error listing files, %s: %s"), pattern, getLastErrorText());
@@ -418,7 +546,7 @@ TCHAR** loggerFileGetFiles(const TCHAR* pattern, int sortMode) {
     }
 #else /* Unix */
 
-#ifdef UNICODE
+ #ifdef UNICODE
     char* cPattern;
     size_t req;
 
@@ -432,9 +560,9 @@ TCHAR** loggerFileGetFiles(const TCHAR* pattern, int sortMode) {
 
     result = glob(cPattern, GLOB_MARK | GLOB_NOSORT, NULL, &g);
     free(cPattern);
-#else
+ #else
     result = glob(pattern, GLOB_MARK | GLOB_NOSORT, NULL, &g);
-#endif
+ #endif
     cnt = 0;
     if (!result) {
         if (g.gl_pathc > 0) {
@@ -455,7 +583,7 @@ TCHAR** loggerFileGetFiles(const TCHAR* pattern, int sortMode) {
             memset(fileTimes, 0, sizeof(time_t) * filesSize);
 
             for (findex = 0; findex < g.gl_pathc; findex++) {
-#ifdef UNICODE
+ #ifdef UNICODE
                 req = mbstowcs(NULL, g.gl_pathv[findex], 0);
                 if (req == (size_t)-1) {
                     invalidMultiByteSequence(TEXT("GLET"), 1);
@@ -469,7 +597,7 @@ TCHAR** loggerFileGetFiles(const TCHAR* pattern, int sortMode) {
                 }
                 mbstowcs(files[cnt], g.gl_pathv[findex], req + 1);
 
-#else
+ #else
                 files[cnt] = malloc((strlen(g.gl_pathv[findex]) + 1));
                 if (!files[cnt]) {
                     outOfMemoryQueued(TEXT("WFGF"), 11);
@@ -479,7 +607,7 @@ TCHAR** loggerFileGetFiles(const TCHAR* pattern, int sortMode) {
                 }
 
                 strncpy(files[cnt], g.gl_pathv[findex], strlen(g.gl_pathv[findex]) + 1);
-#endif
+ #endif
 
                 /* Only try to get the modified time if it is really necessary. */
                 if (sortMode == LOGGER_FILE_SORT_MODE_TIMES) {
@@ -489,15 +617,15 @@ TCHAR** loggerFileGetFiles(const TCHAR* pattern, int sortMode) {
                         log_printf_queue(TRUE, WRAPPER_SOURCE_WRAPPER, LEVEL_WARN, TEXT("Failed to stat %s: %s"), files[cnt], getLastErrorText());
                     }
                 }
-#ifdef WRAPPER_FILE_DEBUG
+ #ifdef WRAPPER_FILE_DEBUG
                 _tprintf(TEXT("  files[%d]=%s, %ld\n"), cnt, files[cnt], fileTimes[cnt]);
-#endif
+ #endif
                 cnt++;
             }
         } else {
-#ifdef WRAPPER_FILE_DEBUG
+ #ifdef WRAPPER_FILE_DEBUG
             printf("  No files matched.\n");
-#endif
+ #endif
             /* No files, but we still need the array. */
             filesSize = 1;
             files = malloc(sizeof(TCHAR *) * filesSize);
@@ -518,9 +646,9 @@ TCHAR** loggerFileGetFiles(const TCHAR* pattern, int sortMode) {
 
         globfree(&g);
     } else if (result == GLOB_NOMATCH) {
-#ifdef WRAPPER_FILE_DEBUG
+ #ifdef WRAPPER_FILE_DEBUG
         _tprintf(TEXT("  No files matched.\n"));
-#endif
+ #endif
         /* No files, but we still need the array. */
         filesSize = 1;
         files = malloc(sizeof(TCHAR *) * filesSize);
@@ -551,6 +679,13 @@ TCHAR** loggerFileGetFiles(const TCHAR* pattern, int sortMode) {
             loggerFileFreeFiles(files);
             return NULL;
         }
+    } else if (sortMode == LOGGER_FILE_SORT_MODE_NAMES_SMART) {
+        if (!sortFilesNamesSmart(pattern, files, cnt)) {
+            /* Failed. Reported. */
+            free(fileTimes);
+            loggerFileFreeFiles(files);
+            return NULL;
+        }
     } else if (sortMode == LOGGER_FILE_SORT_MODE_NAMES_DEC) {
         if (!sortFilesNamesDec(files, cnt)) {
             /* Failed. Reported. */
