@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2020 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -33,7 +33,7 @@
 #include "wrapper_hashmap.h"
 
 #ifndef TRUE
-#define TRUE -1
+#define TRUE 1
 #endif
 
 #ifndef FALSE
@@ -56,6 +56,9 @@
 
 #define PROPERTIES_DUMP_FORMAT_DEFAULT TEXT("SFNV")
 
+#define SECURITY_LEVEL_TRUST    1
+#define SECURITY_LEVEL_CHECK    2
+
 typedef struct EnvSrc EnvSrc;
 struct EnvSrc {
     int     source;                     /* Source of the variable. */
@@ -68,16 +71,20 @@ typedef struct Property Property;
 struct Property {
     TCHAR *name;                        /* The name of the property. */
     TCHAR *value;                       /* The value of the property. */
-    TCHAR *value_inner;                 /* The value of the property (with %WRAPPER_PERCENTAGE% kept unexpanded). */
+    TCHAR *value_inner;                 /* The value with %WRAPPER_PERCENTAGE% kept unexpanded. */
+    TCHAR *value_masked;                /* The value with sensitive data masked. */
     TCHAR *filePath;                    /* The file in which the property was declared. NULL is not defined in a conf file. */
     int lineNumber;                     /* The line number at which the property was declared. 0 if not defined in a conf file. */
-    int definitions;                    /* Counter for the number of time the property was defined. */
-    int finalValue;                     /* TRUE if the Property can not be changed. */
-    int quotable;                       /* TRUE if quotes can be optionally added around the value. */
-    int internal;                       /* TRUE if the Property is internal. */
+    int definitions;                    /* Counter for the number of times the property was defined. */
+    int finalValue;                     /* TRUE if the property can not be changed. */
+    int internal;                       /* TRUE if the property is internal. */
     int isGenerated;                    /* TRUE if the property did not exist in the configuration and was generated using a default value. */
-    int isVariable;                     /* TRUE if the property the definition of a variable (starting with 'set.'). */
+    int isVariable;                     /* TRUE if the property is the definition of a variable (starting with 'set.'). */
+    int expandVars;                     /* TRUE if variables in the value should be expanded. */
+    int isExpanded;                     /* TRUE if all variables have been expanded. */
     int lastDefinitionDepth;            /* Depth of the configuration file in which the property was previously defined, or -1 if not yet defined. */
+    int allowCiphers;                   /* TRUE if this property allows ciphers in the value. */
+    int hasCipher;                      /* TRUE if the value has one or several ciphers. */
     Property *next;                     /* Pointer to the next Property in a linked list */
     Property *previous;                 /* Pointer to the previous Property in a linked list */
 };
@@ -86,16 +93,19 @@ typedef struct Properties Properties;
 struct Properties {
     int debugProperties;                /* TRUE if debug information on Properties should be shown. */
     int exitOnOverwrite;                /* If TRUE, causes the wrapper to exit when any property is overwritten in the config files. */
-    int logLevelOnOverwrite;            /* Defines the log level of the messages reported when properties are overwritten. */
+    int logLevelOnOverwrite;            /* Defines the log level of the messages reported when properties are overwritten (the value may change each time a directive is encountered as the configuration is loading). */
     int overwrittenPropertyCausedExit;  /* Flag to keep trace whether at least one property was overridden */
+    int securityLevel;                  /* Security context when loading properties. Sensitive data can be either: always accepted; allowed only for secure sources; or completely forbidden. */
     int logWarnings;                    /* Flag that controls whether or not warnings will be logged. */
     int logWarningLogLevel;             /* Log level at which any log warnings will be logged. */
     int dumpLogLevel;                   /* Log level at which properties will be dumped. */
     TCHAR* dumpFormat;                  /* Format used when dumping properties. */
+    int preload;                        /* TRUE when pre-loading. */
     Property *first;                    /* Pointer to the first property. */
     Property *last;                     /* Pointer to the last property.  */
     PHashMap warnedVarMap;              /* Map of undefined environment variables for which the user was warned. */
     PHashMap ignoreVarMap;              /* Map of environment variables that should not be expanded. */
+    PHashMap definedVarMap;             /* Map of variable definitions. */
 };
 
 /**
@@ -125,6 +135,15 @@ extern int GetLogLevelOnOverwrite(Properties *properties);
  */
 extern int setEnv(const TCHAR *name, const TCHAR *value, int source);
 
+extern EnvSrc* getEnvSrc(const TCHAR* name);
+
+#define VAR_KEEP            0x0001
+#define VAR_EXPAND          0x0002
+#define VAR_ESCAPEQUOTES    0x0004
+
+#define CIPHER_KEEP         0x0010
+#define CIPHER_FORBID       0x0020
+
 /**
  * Parses a property value and populates any environment variables.  If the expanded
  *  environment variable would result in a string that is longer than bufferLength
@@ -137,11 +156,16 @@ extern int setEnv(const TCHAR *name, const TCHAR *value, int source);
  * @param warnedUndefVarMap Map of variables which have previously been logged, may be NULL if warnUndefinedVars false.
  * @param warnLogLevel Log level at which any warnings will be logged.
  * @param ignoreVarMap Map of environment variables that should not be expanded.
- * @param pHasPercentage Pointer to a variable which will be set to TRUE if a %WRAPPER_PERCENTAGE% variable was found.
- *                         - If a non-NULL pointer is passed, the variable will not be expanded and no warning will be reported.
- *                         - If NULL is passed, the variable will be expanded to '%'.
+ * @param definedVarMap Map of environment variables definitions.
+ * @param isVariableDef TRUE if the value to evaluate is the value of a variable 
+ * @param pHasPercent Pointer to a variable which will be set to TRUE if a %WRAPPER_PERCENTAGE% variable was found.
+ *                      - If a non-NULL pointer is passed, the variable will not be expanded and no warning will be reported.
+ *                      - If NULL is passed, the variable will be expanded to '%'.
+ * @param pHasCipher Pointer to a variable which will be set to TRUE if a cipher %...|...% was found.
+ * @param pAllExpanded Pointer to a variable which will be set to TRUE if all environment variables were expanded after this call.
+ * @param mode A combination of a CIPHER_* constant and a VAR_* constant.
  */
-extern void evaluateEnvironmentVariables(const TCHAR *propertyValue, TCHAR *buffer, int bufferLength, int warnUndefinedVars, PHashMap warnedUndefVarMap, int warnLogLevel, PHashMap ignoreVarMap, int *hasPercentage);
+extern void evaluateEnvironmentVariables(const TCHAR *propertyValue, TCHAR *buffer, size_t bufferLength, int warnUndefinedVars, PHashMap warnedUndefVarMap, int warnLogLevel, PHashMap ignoreVarMap, PHashMap definedVarMap, int isVariableDef, int *pHasPercent, int *pHasCipher, int *pAllExpanded, int mode);
 
 /**
  * This function returns a reference to a static buffer and is NOT thread safe.
@@ -165,6 +189,13 @@ typedef int (*ConfigFileReader_ReadFilterCallbackMB)(const char *bufferMB);
  */
 typedef int (*ConfigFileReader_ReadFilterCallbackW)(const TCHAR *bufferMB);
 
+void disposeParentEnvironmentMap();
+
+int saveParentEnvironment();
+
+void disposeQuotableMap();
+
+int initPropertyLoading(Properties *properties, int preload);
 
 /**
  * Create a Properties structure loaded in from the specified file.
@@ -172,7 +203,7 @@ typedef int (*ConfigFileReader_ReadFilterCallbackW)(const TCHAR *bufferMB);
  *
  * @param properties Properties structure to load into.
  * @param filename File to load the properties from.
- * @param preload TRUE if this is a preload call that should have supressed error output.
+ * @param minLogLevel log level above which messages should be logged.
  * @param originalWorkingDir Working directory of the binary at the moment it was launched.
  * @param fileRequired TRUE if the file specified by filename is required, FALSE if a missing
  *                     file will silently fail.
@@ -180,13 +211,12 @@ typedef int (*ConfigFileReader_ReadFilterCallbackW)(const TCHAR *bufferMB);
  *                           lines that should not be processed.
  *
  * @return CONFIG_FILE_READER_SUCCESS if the file was read successfully,
- *         CONFIG_FILE_READER_OPEN_FAIL if the file could not be found or opened.
- *         CONFIG_FILE_READER_FAIL if there were any problems at all, or
- *         CONFIG_FILE_READER_HARD_FAIL if the problem should cascaded all the way up.
+ *         CONFIG_FILE_READER_OPEN_FAIL if the file could not be found or opened,
+ *         CONFIG_FILE_READER_FAIL if an error has occurred and loading should stop.
  */
 extern int loadProperties(Properties *properties,
                           const TCHAR* filename,
-                          int preload,
+                          int minLogLevel,
                           const TCHAR *originalWorkingDir,
                           int fileRequired,
                           ConfigFileReader_ReadFilterCallbackMB readFilterCallback);
@@ -195,7 +225,7 @@ extern int loadProperties(Properties *properties,
  * Create a Properties structure.  Must call disposeProperties to free up
  *  allocated memory.
  */
-extern Properties* createProperties(int debug, int logLevelOnOverwrite, int exitOnOverwrite);
+extern Properties* createProperties(int debug, int logLevelOnOverwrite, int exitOnOverwrite, int securityLevel);
 
 /**
  * Free all memory allocated by a Properties structure.  The properties
@@ -217,6 +247,15 @@ extern void disposeEnvironment();
  */
 extern int removeProperty(Properties *properties, const TCHAR *propertyName);
 
+/**
+ * Used to set a NULL terminated list of property names that cannot be reloaded.
+ *
+ * @param propertyNames NULL terminated list of property names.  Property names
+ *                      can contain a single '*' wildcard which will match 0 or
+ *                      more characters.
+ */
+extern void setStickyProperties(const TCHAR **propertyNames);
+
 /**
  * Used to set a NULL terminated list of property names whose values should be
  *  escaped when read in from a file.   '\\' will become '\' and '\n' will
@@ -229,14 +268,33 @@ extern int removeProperty(Properties *properties, const TCHAR *propertyName);
 extern void setEscapedProperties(const TCHAR **propertyNames);
 
 /**
- * Returns true if the specified property matches one of the property names
- *  previously set in a call to setEscapedProperties()
+ * Returns the property value with sensitive data masked.
  *
- * @param propertyName Property name to test.
+ * @param properties Properties structure.
+ * @param propertyName Name of the property.
  *
- * @return TRUE if the property should be escaped.  FALSE otherwise.
+ * @return masked value
  */
-extern int isEscapedProperty(const TCHAR *propertyName);
+extern const TCHAR* getMaskedValueOfProperty(Properties *properties, const TCHAR* propertyName);
+
+/**
+ * Returns true if the value of the specified property contains sensitive data.
+ *
+ * @param properties Properties structure.
+ * @param propertyName Name of the property.
+ *
+ * @return TRUE if the property value contains sensitive data, FALSE otherwise.
+ */
+extern int isSensitiveProperty(Properties *properties, const TCHAR* propertyName);
+
+/**
+ * Return TRUE if the value of the property should be displayed as '<hidden>' when printed in the logs.
+ *
+ * @property property to check.
+ *
+ * @return TRUE if the value should be hidden.
+ */
+extern int isPasswordProperty(Property *property);
 
 /**
  * Adds a single property to the properties structure.
@@ -248,30 +306,28 @@ extern int isEscapedProperty(const TCHAR *propertyName);
  * @param propertyName Name of the new Property.
  * @param propertyValue Initial property value.
  * @param finalValue TRUE if the property should be set as static.
- * @param quotable TRUE if the property could contain quotes.
  * @param escapable TRUE if the propertyValue can be escaped if its propertyName
  *                  is in the list set with setEscapedProperties().
+ * @param expandVars TRUE if variable in propertyValue should be expanded.
  * @param internal TRUE if the property is a Wrapper internal property.
  *
  * @return The newly created Property, or NULL if there was a reported error.
  */
-extern Property* addProperty(Properties *properties, const TCHAR* filename, int lineNum, int depth, const TCHAR *propertyName, const TCHAR *propertyValue, int finalValue, int quotable, int escapable, int internal);
+extern Property* addProperty(Properties *properties, const TCHAR* filename, int lineNum, int depth, const TCHAR *propertyName, const TCHAR *propertyValue, int finalValue, int escapable, int expandVars, int internal);
 
 /**
  * Takes a name/value pair in the form <name>=<value> and attempts to add
  * it to the specified properties table.
  *
  * @param properties Properties structure to add to.
- * @param filename Name of the file from which the property was loaded.  NULL, if not from a file.
- * @param lineNum Line number of the property declaration in the file.  Ignored if filename is NULL.
  * @param propertyNameValue The "name=value" pair to create the property from.
  * @param finalValue TRUE if the property should be set as static.
- * @param quotable TRUE if the property could contain quotes.
  * @param internal TRUE if the property is a Wrapper internal property.
+ * @param pHasCipher Pointer to a variable which will be set to TRUE if a cipher %...|...% was found.
  *
  * Returns 0 if successful, otherwise 1
  */
-extern int addPropertyPair(Properties *properties, const TCHAR* filename, int lineNum, const TCHAR *propertyNameValue, int finalValue, int quotable, int internal);
+extern int addPropertyPair(Properties *properties, const TCHAR *propertyNameValue, int finalValue, int internal, int* pHasCipher);
 
 extern void setInternalVarProperty(Properties *properties, const TCHAR *varName, const TCHAR *varValue, int finalValue, int ignore);
 
@@ -321,9 +377,11 @@ extern int getBooleanProperties(Properties *properties, const TCHAR *propertyNam
 
 extern void freeBooleanProperties(TCHAR **propertyNames, int *propertyValues, long unsigned int *propertyIndices);
 
-extern int isGeneratedProperty(Properties *properties, const TCHAR *propertyName);
+extern const TCHAR* getPropertyFilePath(Properties *properties, const TCHAR* propertyName);
 
-extern int isQuotableProperty(Properties *properties, const TCHAR *propertyName);
+extern int getStatusProperty(Properties *properties, const TCHAR *propertyName, int defaultValue);
+
+extern int isGeneratedProperty(Properties *properties, const TCHAR *propertyName);
 
 extern void dumpProperties(Properties *properties);
 
@@ -347,6 +405,8 @@ extern void setLogPropertyWarnings(Properties *properties, int logWarnings);
  */
 extern void setLogPropertyWarningLogLevel(Properties *properties, int logLevel);
 
+extern void adjustStickyProperties(Properties *tgt, Properties *src, int addOnly);
+
 /**
  * Returns the minimum value. This is used in place of the __min macro when the parameters should not be called more than once.
  */
@@ -359,7 +419,5 @@ extern int propIntMax(int value1, int value2);
 
 /** Creates a linearized representation of all of the properties.
  *  The returned buffer must be freed by the calling code. */
-extern TCHAR *linearizeProperties(Properties *properties, TCHAR separator);
-
+extern TCHAR *linearizeProperties(Properties *properties, TCHAR separator, int stripSensitiveProperties, int stripInternalProperties);
 #endif
-
