@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2018 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -30,12 +30,8 @@
 #include "wrapperinfo.h"
 #endif
 
-#if defined(IRIX)
- #define PATH_MAX FILENAME_MAX
-#endif
-
 #ifndef TRUE
-#define TRUE -1
+#define TRUE 1
 #endif
 
 #ifndef FALSE
@@ -511,24 +507,24 @@ TCHAR *_tsetlocale(int category, const TCHAR *locale) {
     return NULL;
 }
 
-int createWideFormat(const wchar_t *fmt, wchar_t **msg) {
+int createWideFormat(const wchar_t *fmt, wchar_t **wFmt) {
     int i, result;
     
     if (wcsstr(fmt, TEXT("%s")) != NULL) {
-        *msg = malloc(sizeof(wchar_t) * (wcslen(fmt) + 1));
-        if (*msg) {
-            wcsncpy(*msg, fmt, wcslen(fmt) + 1);
+        *wFmt = malloc(sizeof(wchar_t) * (wcslen(fmt) + 1));
+        if (*wFmt) {
+            wcsncpy(*wFmt, fmt, wcslen(fmt) + 1);
             for (i = 0; i < wcslen(fmt); i++){
                 if (fmt[i] == TEXT('%') && i  < wcslen(fmt) && fmt[i + 1] == TEXT('s') && (i == 0 || fmt[i - 1] != TEXT('%'))) {
-                    (*msg)[i + 1] = TEXT('S');
+                    (*wFmt)[i + 1] = TEXT('S');
                     i++;
                 }
             }
-            (*msg)[wcslen(fmt)] = TEXT('\0');
+            (*wFmt)[wcslen(fmt)] = TEXT('\0');
         }
         result = TRUE;
     } else {
-        *msg = (wchar_t*)fmt;
+        *wFmt = (wchar_t*)fmt;
         result = FALSE;
     }
     return result;
@@ -536,16 +532,16 @@ int createWideFormat(const wchar_t *fmt, wchar_t **msg) {
 
 int _tprintf(const wchar_t *fmt,...) {
     int i, flag;
-    wchar_t *msg = NULL;
+    wchar_t *wFmt = NULL;
     va_list args;
 
-    flag = createWideFormat(fmt, &msg);
-    if (msg) {
+    flag = createWideFormat(fmt, &wFmt);
+    if (wFmt) {
         va_start(args, fmt);
-        i = vwprintf(msg, args);
+        i = vwprintf(wFmt, args);
         va_end (args);
-        if (flag == TRUE) {
-            free(msg);
+        if (flag) {
+            free(wFmt);
         }
         return i;
     }
@@ -554,16 +550,16 @@ int _tprintf(const wchar_t *fmt,...) {
 
 int _ftprintf(FILE *stream, const wchar_t *fmt, ...) {
     int i, flag;
-    wchar_t *msg = NULL;
+    wchar_t *wFmt = NULL;
     va_list args;
 
-    flag = createWideFormat(fmt, &msg);
-    if (msg) {
+    flag = createWideFormat(fmt, &wFmt);
+    if (wFmt) {
         va_start(args, fmt);
-        i = vfwprintf(stream, msg, args);
+        i = vfwprintf(stream, wFmt, args);
         va_end (args);
-        if (flag == TRUE) {
-            free(msg);
+        if (flag) {
+            free(wFmt);
         }
         return i;
     }
@@ -572,16 +568,16 @@ int _ftprintf(FILE *stream, const wchar_t *fmt, ...) {
 
 int _sntprintf(TCHAR *str, size_t size, const TCHAR *fmt, ...) {
     int i, flag;
-    wchar_t *msg = NULL;
+    wchar_t *wFmt = NULL;
     va_list args;
 
-    flag = createWideFormat(fmt, &msg);
-    if (msg) {
+    flag = createWideFormat(fmt, &wFmt);
+    if (wFmt) {
         va_start(args, fmt);
-        i = vswprintf(str, size, msg, args);
+        i = vswprintf(str, size, wFmt, args);
         va_end (args);
-        if (flag == TRUE) {
-            free(msg);
+        if (flag) {
+            free(wFmt);
         }
         return i;
     }
@@ -858,7 +854,7 @@ int _texecvp(TCHAR* arg, TCHAR **cmd) {
                 return -1;
             }
         }
-        cCmd[size] = '\0';
+        cCmd[size] = NULL;
 
         req = wcstombs(NULL, arg, 0);
         if (req == (size_t)-1) {
@@ -953,7 +949,7 @@ int _texecve(TCHAR* arg, TCHAR **cmd, TCHAR** env) {
                 return -1;
             }
         }
-        cCmd[sizeCmd] = '\0';
+        cCmd[sizeCmd] = NULL;
         for (i = 0; env[i] != NULL; i++) {
             ;
         }
@@ -997,7 +993,7 @@ int _texecve(TCHAR* arg, TCHAR **cmd, TCHAR** env) {
                 return -1;
             }
         }
-        cEnv[sizeEnv] = '\0';
+        cEnv[sizeEnv] = NULL;
 
         req  = wcstombs(NULL, arg, 0);
         if (req == (size_t)-1) {
@@ -1151,6 +1147,26 @@ int _tstat(const wchar_t* filename, struct stat *buf) {
     return size;
 }
 
+int _tchown(const TCHAR *path, uid_t owner, gid_t group) {
+    char* cPath;
+    int r;
+    size_t size;
+
+    size = wcstombs(NULL, path, 0);
+    if (size == (size_t)-1) {
+        return -1;
+    }
+
+    cPath = malloc(size + 1);
+    if (cPath) {
+        wcstombs(cPath, path, size + 1);
+        r = chown(cPath, owner, group);
+        free(cPath);
+        return r;
+    }
+    return -1;
+}
+
 /**
  * Expands symlinks and resolves /./, /../ and extra '/' characters to produce a
  *  canonicalized absolute pathname.
@@ -1166,11 +1182,7 @@ int _tstat(const wchar_t* filename, struct stat *buf) {
  */
 wchar_t* _trealpathN(const wchar_t* fileName, wchar_t *resolvedName, size_t resolvedNameSize) {
     char *cFile;
-#if defined(IRIX)
-    char resolved[FILENAME_MAX + 1];
-#else
     char resolved[PATH_MAX + 1];
-#endif
     int sizeFile;
     int req;
     char* returnVal;
@@ -1220,6 +1232,113 @@ wchar_t* _trealpathN(const wchar_t* fileName, wchar_t *resolvedName, size_t reso
     }
     return NULL;
 }
+
+ #if !defined(LINUX) && !defined(AIX)
+/**
+ * This function is not defined on some Unix platforms, so lets define it.
+ */
+int _tcsnicmp(const wchar_t *string1, const wchar_t *string2, size_t count) {
+    wint_t c1, c2;
+
+    if ((string1 == string2) || (count == 0)) {
+        return 0;
+    }
+
+    do {
+        c1 = (wint_t)towlower(*string1++);
+        c2 = (wint_t)towlower(*string2++);
+        if ((c1 == 0) || (c1 != c2)) {
+            return c1 - c2;
+        }
+    } while (--count > 0);
+
+    return c1 - c2;
+}
+ #endif
+#endif
+
+/**
+ * Fill a block of memory with zeros.
+ *  Use this function to ensure no sensitive data remains in memory.
+ *
+ * @param str A pointer to the starting address of the block of memory to fill with zeros.
+ * @param size The size of the block of memory to fill with zeros, in bytes.
+ */
+void wrapperSecureZero(void* str, size_t size) {
+    if (str && (size > 0)) {
+#ifdef WIN32
+        SecureZeroMemory(str, size);
+#else
+        memset (str, '\0', size);
+ #ifdef __GNUC__
+        /* Compiler barrier.  */
+        asm volatile ("" ::: "memory");
+ #endif
+#endif
+    }
+}
+
+/**
+ * Fill a block of memory with zeros, then free it.
+ *  Use this function to ensure no sensitive data remains in memory.
+ *
+ * @param str A pointer to the starting address of the block of memory to fill with zeros.
+ * @param size The size of the block of memory to fill with zeros, in bytes.
+ */
+void wrapperSecureFree(void* str, size_t size) {
+    wrapperSecureZero(str, size);
+    free(str);
+}
+
+/**
+ * Fill a Wide-char sequence with zeros, then free it.
+ *  Use this function to ensure no sensitive data remains in memory.
+ *
+ * @param str String to erase and free.
+ */
+void wrapperSecureFreeStrW(TCHAR* str) {
+    if (str) {
+        wrapperSecureFree(str, _tcslen(str) * sizeof(TCHAR));
+    }
+}
+
+/**
+ * Fill a multi-byte sequence with zeros, then free it.
+ *  Use this function to ensure no sensitive data remains in memory.
+ *
+ * @param str String to erase and free.
+ */
+void wrapperSecureFreeStrMB(char* str) {
+    if (str) {
+        wrapperSecureFree(str, strlen(str));
+    }
+}
+
+#ifdef USE_USLEEP
+ #define USLEEP_MAX (1000000 - 1)
+
+/**
+ * Wrapper function for usleep() allowing to suspends thread execution for more than 1 second.
+ *  According to the doc, usec must be smaller than 1000000 on some systems (this is the case on z/OS).
+ *  This function splits the suspension time into intervals no longer than USLEEP_MAX and repeat as
+ *  many times as needed.
+ *
+ * @param usec suspension time in microsoconds
+ *
+ * @return 0 on success, -1 on error.
+ */
+int wrapperUsleep(unsigned long usec) {
+    unsigned long chunk;
+    
+    while(usec > 0) {
+        chunk = usec > USLEEP_MAX ? USLEEP_MAX : usec;
+        if (usleep(chunk) == -1) {
+            return -1;
+        }
+        usec -= chunk;
+    }
+    return 0;
+}
 #endif
 
 /**
@@ -1276,6 +1395,29 @@ TCHAR* toUpper(const TCHAR* value) {
     return result;
 }
 
+/**
+ * Updates a string value by making a copy of the original.  Any old value is
+ *  first freed.
+ */
+int updateStringValue(TCHAR **ptr, const TCHAR *value) {
+    if (*ptr != NULL) {
+        free(*ptr);
+        *ptr = NULL;
+    }
+
+    if (value != NULL) {
+        *ptr = malloc(sizeof(TCHAR) * (_tcslen(value) + 1));
+        if (!(*ptr)) {
+            outOfMemory(TEXT("USV"), 1);
+            /* TODO: This is pretty bad.  Not sure how to recover... */
+            return TRUE;
+        } else {
+            _tcsncpy(*ptr, value, _tcslen(value) + 1);
+        }
+    }
+    return FALSE;
+}
+
 /**
  * Clear any non-alphanumeric characters.
  *  Generally the OS will ignore the canonical dashes and punctuation in the encoding notation when setting the locale.
@@ -1284,7 +1426,7 @@ TCHAR* toUpper(const TCHAR* value) {
  * @param bufferIn input string
  * @param bufferOut output string
  */
-void clearNonAlphanumeric(TCHAR* bufferIn, TCHAR* bufferOut) {
+void clearNonAlphanumeric(const TCHAR* bufferIn, TCHAR* bufferOut) {
     while (*bufferIn) {
         if (_istdigit(*bufferIn) || _istalpha(*bufferIn)) {
             *bufferOut = *bufferIn;
@@ -1296,6 +1438,94 @@ void clearNonAlphanumeric(TCHAR* bufferIn, TCHAR* bufferOut) {
 }
 
 #ifndef WIN32
+ #ifdef LINUX
+/**
+ * Check if the encoding is specified with the canonical name.
+ *  Ex: 'UTF-8' is canonical, 'utf8' isn't.
+ *
+ * @param encoding
+ *
+ * @return TRUE if this is a canonical name, FALSE otherwise.
+ */
+static int encodingIsCanonicalName(const TCHAR* encoding) {
+    TCHAR c;
+    int i;
+
+    for (i = 0; i < _tcslen(encoding); i++) {
+        c = encoding[i];
+        if (c >= TEXT('A') && c <= TEXT('Z')) {
+            return TRUE;
+        }
+        if (c == TEXT('-')) {
+            return TRUE;
+        }
+    }
+    return FALSE;
+}
+ #endif
+
+/**
+ * Compares two encodings.
+ *
+ * @param encoding1 When using systemMode, this should be the system encoding
+ * @param encoding2
+ * @param ignoreCase TRUE if the case should be ignored
+ *                   TRUE if dashes and punctuation should be ignored.
+ *
+ * @return TRUE if the encodings are identical, FALSE otherwise.
+ */
+int compareEncodings(const TCHAR* encoding1, const TCHAR* encoding2, int ignoreCase, int ignorePunctuation) {
+    TCHAR encoding1Buff[ENCODING_BUFFER_SIZE];
+    TCHAR encoding2Buff[ENCODING_BUFFER_SIZE];
+    const TCHAR *enc1Ptr;
+    const TCHAR *enc2Ptr;
+    
+    if (encoding1 && encoding2) {
+        if (ignorePunctuation) {
+            clearNonAlphanumeric(encoding1, encoding1Buff);
+            clearNonAlphanumeric(encoding2, encoding2Buff);
+            enc1Ptr = encoding1Buff;
+            enc2Ptr = encoding2Buff;
+        } else {
+            enc1Ptr = encoding1;
+            enc2Ptr = encoding2;
+        }
+        if (ignoreCase) {
+            return (strcmpIgnoreCase(enc1Ptr, enc2Ptr) == 0);
+        } else {
+            return (_tcscmp(enc1Ptr, enc2Ptr) == 0);
+        }
+    }
+    return (!encoding1 && !encoding2);
+}
+
+/**
+ * Compares two encodings with the rules of the OS.
+ *  On Linux the comparison ignores case, dashes and punctuation 
+ *   (except when the encoding of the system locale is displayed with a canonical name, e.g. C.UTF-8).
+ *  On HPUX, Solaris, AIX and FreeBSD, the comparison is strict.
+ *  On MAC and zOS, the comparison ignores case, but is strict regarding dashes and punctuation.
+ *
+ * @param encoding1 system encoding
+ * @param encoding2 other encoding
+ *
+ * @return TRUE if the encodings are identical, FALSE otherwise.
+ */
+int compareEncodingsSysMode(const TCHAR* encoding1, const TCHAR* encoding2) {
+    int ignoreCase = FALSE;
+    int ignorePunctuation = FALSE;
+
+ #ifdef LINUX
+    if (!encodingIsCanonicalName(encoding1)) {
+        ignoreCase = TRUE;
+        ignorePunctuation = TRUE;
+    }
+ #elif defined(MACOSX) || defined(ZOS)
+    ignoreCase = TRUE;
+ #endif
+    return compareEncodings(encoding1, encoding2, ignoreCase, ignorePunctuation);
+}
+
 /**
  * Get the encoding of the current locale.
  *
@@ -1555,7 +1785,7 @@ int converterWideToMB(const TCHAR *wideChars, char **outputBufferMB, int outputE
         errorTemplateLen = strlen(errorTemplate) + 10 + 1;
         *outputBufferMB = malloc(errorTemplateLen);
         if (*outputBufferMB) {
-            _snprintf(*outputBufferMB, errorTemplateLen, errorTemplate, wrapperGetLastError());
+            _snprintf(*outputBufferMB, errorTemplateLen, errorTemplate, GetLastError());
         } else {
             /* Out of memory. *outputBufferW already NULL. */
         }
@@ -1890,11 +2120,11 @@ int converterWideToMB(const TCHAR *wideChars, char **outputBufferMB, const char
     len = wcstombs(NULL, wideChars, 0);
 
     if (len == (size_t)-1) {
-        errorTemplate = "Invalid multibyte sequence (0x%x)";
-        errorTemplateLen = strlen(errorTemplate) + 10 + 1;
+        errorTemplate = "Invalid multibyte sequence";
+        errorTemplateLen = strlen(errorTemplate) + 1;
         *outputBufferMB = malloc(errorTemplateLen);
         if (*outputBufferMB) {
-            snprintf(*outputBufferMB, errorTemplateLen, errorTemplate, wrapperGetLastError());
+            strncpy(*outputBufferMB, errorTemplate, errorTemplateLen);
         } else {
             /* Out of memory. *outputBufferW already NULL. */
         }
@@ -1926,9 +2156,9 @@ int converterWideToMB(const TCHAR *wideChars, char **outputBufferMB, const char
 #endif
 
 /**
- * Gets the error code for the last operation that failed.
+ * Gets the error code for the last socket operation that failed.
  */
-int wrapperGetLastError() {
+int wrapperGetSocketLastError() {
 #ifdef WIN32
     return WSAGetLastError();
 #else
@@ -1936,13 +2166,16 @@ int wrapperGetLastError() {
 #endif
 }
 
-/*
+/**
  * Corrects a path in place by replacing all '/' characters with '\'
  *  on Windows platforms.  Does nothing on NIX platforms.
  *
  * filename - Filename to be modified.  Could be null.
+ *
+ * @return TRUE if the filename was changed, FALSE otherwise.
  */
-void wrapperCorrectWindowsPath(TCHAR *filename) {
+int wrapperCorrectWindowsPath(TCHAR *filename) {
+    int result = FALSE;
 #ifdef WIN32
     TCHAR *c;
 
@@ -1950,18 +2183,23 @@ void wrapperCorrectWindowsPath(TCHAR *filename) {
         c = (TCHAR *)filename;
         while((c = _tcschr(c, TEXT('/'))) != NULL) {
             c[0] = TEXT('\\');
+            result = TRUE;
         }
     }
 #endif
+    return result;
 }
 
-/*
+/**
  * Corrects a path in place by replacing all '\' characters with '/'
  *  on NIX platforms.  Does nothing on Windows platforms.
  *
  * filename - Filename to be modified.  Could be null.
+ *
+ * @return TRUE if the filename was changed, FALSE otherwise.
  */
-void wrapperCorrectNixPath(TCHAR *filename) {
+int wrapperCorrectNixPath(TCHAR *filename) {
+    int result = FALSE;
 #ifndef WIN32
     TCHAR *c;
 
@@ -1969,9 +2207,11 @@ void wrapperCorrectNixPath(TCHAR *filename) {
         c = (TCHAR *)filename;
         while((c = _tcschr(c, TEXT('\\'))) != NULL) {
             c[0] = TEXT('/');
+            result = TRUE;
         }
     }
 #endif
+    return result;
 }
 
 #ifndef WIN32
@@ -2159,7 +2399,7 @@ void *malloc2(size_t size, const char *file, int line, const char *func, const c
     void *ptr;
  #ifdef WIN32
     wprintf(L"%S:%d:%S malloc(%S) -> malloc(%d)", file, line, func, sizeVar, size);
- #else	
+ #else
     wprintf(L"%s:%d:%s malloc(%s) -> malloc(%d)", file, line, func, sizeVar, size);
  #endif
     ptr = malloc(size);
