@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2020 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -143,7 +143,11 @@ extern wchar_t* _trealpathN(const wchar_t* fileName, wchar_t *resolvedName, size
 #define _tcsnccoll    _wcsncoll
 #define _tcsnextc     _wcsnextc
 #define _tcsncicmp    _wcsnicmp
-#define _tcsnicmp     _wcsnicmp
+#if defined(LINUX) || defined(AIX)
+ #define _tcsnicmp    wcsncasecmp
+#else
+extern int _tcsnicmp(const wchar_t *string1, const wchar_t *string2, size_t count);
+#endif
 #define _tcsncicoll   _wcsnicoll
 #define _tcsnicoll    _wcsnicoll
 #define _tcsninc      _wcsninc
@@ -345,7 +349,7 @@ typedef unsigned char _TUCHAR;
 #define _tcsnccoll    _strncoll
 #define _tcsnextc     _strnextc
 #define _tcsncicmp    _strnicmp
-#define _tcsnicmp     _strnicmp
+#define _tcsnicmp     strncasecmp
 #define _tcsncicoll   _strnicoll
 #define _tcsnicoll    _strnicoll
 #define _tcsninc      _strninc
@@ -455,21 +459,34 @@ typedef unsigned char _TUCHAR;
 #include <sys/types.h>
 #include <sys/stat.h>
 extern int multiByteToWideChar(const char *multiByteChars, int encoding, TCHAR **outputBufferW, int localizeErrorMessage);
+ #define converterMBToWide(m, e, o, l)  multiByteToWideChar(m, e, o, l)
 #endif
+
+#define COUNT_DIGITS(n)     ((int)(n == 0 ? 1 : (n < 0 ? 1 : 0) + floor(log10(abs(n))) + 1))
+
 /* Define boolean constants. */
 #ifndef TRUE
- #define TRUE -1
+ #define TRUE 1
 #endif
 
 #ifndef FALSE
  #define FALSE 0
 #endif
 
+#define B2S(b) ((b) ? TEXT("TRUE") : TEXT("FALSE"))
+
+#define STATUS_UNCHANGED  0
+#define STATUS_ENABLED    1
+#define STATUS_DISABLED   2
+#define STATUS_UNKNOWN    3
+
 #ifndef WIN32
  #ifdef HPUX
   #define MB_UTF8   "utf8"
+  #define W_UTF8    TEXT("utf8")
  #else
   #define MB_UTF8   "UTF-8"
+  #define W_UTF8    TEXT("UTF-8")
  #endif
  #define __UTF8     MB_UTF8
 #else
@@ -496,15 +513,21 @@ void wrapperSecureFreeStrW(TCHAR* str);
 
 void wrapperSecureFreeStrMB(char* str);
 
+#ifdef USE_USLEEP
+int wrapperUsleep(unsigned long usec);
+#endif
+
 extern TCHAR* toLower(const TCHAR* value);
 
 extern TCHAR* toUpper(const TCHAR* value);
 
-extern void clearNonAlphanumeric(TCHAR* bufferIn, TCHAR* bufferOut);
+extern int updateStringValue(TCHAR **ptr, const TCHAR *value);
 
-extern int compareEncodings(TCHAR* encoding1, TCHAR* encoding2, int ignoreCase, int ignorePunctuation);
+extern void clearNonAlphanumeric(const TCHAR* bufferIn, TCHAR* bufferOut);
 
-extern int compareEncodingsSysMode(TCHAR* encoding1, TCHAR* encoding2);
+extern int compareEncodings(const TCHAR* encoding1, const TCHAR* encoding2, int ignoreCase, int ignorePunctuation);
+
+extern int compareEncodingsSysMode(const TCHAR* encoding1, const TCHAR* encoding2);
 
 #ifndef WIN32
 /**
@@ -658,9 +681,9 @@ extern int converterWideToMB(const TCHAR *wideChars, char **outputBufferMB, cons
 #endif
 
 /**
- * Gets the error code for the last operation that failed.
+ * Gets the error code for the last socket operation that failed.
  */
-extern int wrapperGetLastError();
+extern int wrapperGetSocketLastError();
 
 /*
  * Corrects a path in place by replacing all '/' characters with '\'
