@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2019 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -36,6 +36,7 @@
 #endif
 #include "logger.h"
 #include "wrapper.h"
+#include "wrapperinfo.h"
 #include "wrapper_jvminfo.h"
 #include "property.h"
 
@@ -84,7 +85,7 @@ void disposeJavaVersion(JavaVersion *javaVersion) {
 int parseJavaVersionInner(const TCHAR *javaVersionStr, unsigned int *major, unsigned int *minor, unsigned int *revision, unsigned int defaultComponentValue) {
     TCHAR c[16]; /* This size should be enough to get the version on the first line of the output. */
     TCHAR *ptr;
-    TCHAR *endptr;
+    TCHAR *endptr = NULL;
     TCHAR *token;
 #if defined(UNICODE) && !defined(WIN32)
     TCHAR *state = NULL;
@@ -155,10 +156,8 @@ int parseJavaVersionInner(const TCHAR *javaVersionStr, unsigned int *major, unsi
             }
             errno = 0;
             *revision = (unsigned int)_tcstoul(token, &endptr, 10);
-            /* This time it is ok if some additional characters remain (*endptr not NULL).
-             *  For example on HPUX-ia, the 'java -version' output is "1.8.0.14-hp-ux". */
-            if ((errno != 0) || (token == endptr)) {
-                /* invalid || no digits were read */
+            if ((errno != 0) || (token == endptr) || (*endptr)) {
+                /* invalid || no digits were read || additional characters remain */
                 return TRUE;
             }
         } else {
@@ -200,6 +199,22 @@ JavaVersion* parseJavaVersion(const TCHAR *javaVersionStr, unsigned int defaultC
     return result;
 }
 
+/**
+ * Compare a Java version upon a given string and return TRUE if it is greater or equal, FALSE otherwise.
+ */
+int isJavaGreaterOrEqual(JavaVersion* currentJavaVersion, const TCHAR* javaVersionStr) {
+    JavaVersion* refJavaVersion = parseJavaVersion(javaVersionStr, 0);
+    int result = FALSE;
+
+    if (refJavaVersion) {
+        if (compareJavaVersion(currentJavaVersion, refJavaVersion) >= 0) {
+            result = TRUE;
+        }
+        disposeJavaVersion(refJavaVersion);
+    }
+    return result;
+}
+
 /**
  * Get the Java version from a configuration property.
  *
@@ -249,10 +264,8 @@ JavaVersion* getMinRequiredJavaVersion() {
     JavaVersion* result = createJavaVersion();
     
     if (result) {
-        result->major = 4;
-        result->minor = 0;
-        result->revision = 0;
-        updateStringValue(&result->displayName, TEXT("1.4"));
+        parseJavaVersionInner(wrapperJavacTargetVersion, &(result->major), &(result->minor), &(result->revision), 0);
+        updateStringValue(&result->displayName, wrapperJavacTargetVersion);
     }
     return result;
 }
@@ -308,6 +321,16 @@ int parseOutputJavaVersionInner(TCHAR *javaOutput, TCHAR **pJavaVersionStr, unsi
         return TRUE;
     }
     *ptr = 0;
+    ptr = *pJavaVersionStr;
+    while (*ptr) {
+        if (!_istdigit(*ptr) && (*ptr != '.') && (*ptr != '_')) {
+            /* Some JVM implementations (or customized JVMs) append a label after the version.
+             *  Examples: "1.8.0.14-hp-ux", "9-Raspbian". */
+            *ptr = 0;
+            break;
+        }
+        ptr++;
+    }
 
     return parseJavaVersionInner(*pJavaVersionStr, major, minor, revision, 0);
 }
@@ -441,7 +464,7 @@ const TCHAR* getJvmVendorName(int jvmVendor) {
  *              JVM_BITS_32
  *              JVM_BITS_UNKNOWN
  */
-int parseOutputJvmBits(TCHAR* output) {
+int parseOutputJvmBits(TCHAR* output, JavaVersion *optionalJavaVersion) {
     if (output) {
         /* This is experimental and may not work for all JVMs.
          * For example a IBM JVM will return the following output:
@@ -452,11 +475,19 @@ int parseOutputJvmBits(TCHAR* output) {
             _tcsstr(output, TEXT("64 Bit")) ||
             _tcsstr(output, TEXT("64 bit")) ||
             _tcsstr(output, TEXT("-64 ")) ||
-            _tcsstr(output, TEXT("ppc64")) ||
-            _tcsstr(output, TEXT("mixed mode"))) {
+            _tcsstr(output, TEXT("ppc64"))) {
             return JVM_BITS_64;
         } else {
+#if defined(HPUX) || defined(MACOSX) || defined(SOLARIS) || defined(FREEBSD)
+            /* On these systems, the JVM (for version < 9) can operate both in 32-bit and 64-bits.
+             *  To confirm the bits, we would need to first run 'java -version', then if and only if
+             *  the version is less than 9, run 'java -d64 -version' (-d64 is invalid for Java 9+). */
+            if ((optionalJavaVersion == NULL) || (optionalJavaVersion->major < 9)) {
+                return JVM_BITS_UNKNOWN;
+            }
+#else
             return JVM_BITS_32;
+#endif
         }
     }
     return JVM_BITS_UNKNOWN;
