@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2018 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -63,6 +63,9 @@ int redirectedStdOut = FALSE;
 /** Flag to keep track of whether StdErr has been redirected. */
 int redirectedStdErr = FALSE;
 
+/** Flag to enable tests methods. */
+static int testsDisabled = TRUE;
+
 /* Special symbols that need to be defined manually as part of the bootstrap process. */
 const char utf8ClassJavaLangString[] = {106, 97,118, 97, 47, /* java/ */
                                         108, 97,110,103, 47, /* lang/ */
@@ -89,9 +92,14 @@ char *utf8SigVrLjavaUtilProperties;
 char *utf8MethodGetProperty;
 char *utf8SigLjavaLangStringrLjavaLangString;
 char *utf8javaIOIOException; /* "java/io/IOException" */
+char *utf8SigZ;
 
 #ifdef WIN32
 #else
+char *utf8javaioFileDescriptor;
+char *utf8fd;
+char *utf8SigI;
+char *utf8VrV;
 char *utf8ClassOrgTanukisoftwareWrapperWrapperUNIXUser;
 char *utf8MethodSetGroup;
 char *utf8MethodAddGroup;
@@ -100,45 +108,54 @@ char *utf8SigIStringrV;
 #endif
 
 
-
 /**
  * Cause the current thread to sleep for the specified number of milliseconds.
- *  Sleeps over one second are not allowed.
  *
  * @param ms Number of milliseconds to wait for.
+ * @param interrupt TRUE to return when nanosleep was interrupted by a signal,
+ *                  FALSE to continue sleeping the remaining time.
+ *                  This parameter is ignored on Windows or when using usleep.
  *
- * @return TRUE if the was interrupted, FALSE otherwise.  Neither is an error.
+ * @return the number of remaining ms to sleep if interrupted, -1 if there was an error, 0 otherwise.
  */
-int wrapperSleep(int ms) {
+int wrapperSleepInterrupt(int ms, int interrupt) {
+    int result;
+
 #ifdef WIN32
     Sleep(ms);
+    result = 0; /* On Windows Sleep returns void. */
 #else
     /* We want to use nanosleep if it is available, but make it possible for the
        user to build a version that uses usleep if they want.
        usleep does not behave nicely with signals thrown while sleeping.  This
        was the believed cause of a hang experienced on one Solaris system. */
-#ifdef USE_USLEEP
-    usleep(ms * 1000); /* microseconds */
-#else
+ #ifdef USE_USLEEP
+    result = wrapperUsleep(ms * 1000); /* microseconds */
+ #else
     struct timespec ts;
+    struct timespec tsRemaining;
 
     if (ms >= 1000) {
-        ts.tv_sec = (ms * 1000000) / 1000000000;
-        ts.tv_nsec = (ms * 1000000) % 1000000000; /* nanoseconds */
+        ts.tv_sec = ms / 1000;
+        ts.tv_nsec = (ms % 1000) * 1000000; /* nanoseconds */
     } else {
         ts.tv_sec = 0;
         ts.tv_nsec = ms * 1000000; /* nanoseconds */
     }
-    if (nanosleep(&ts, NULL)) {
-        if (errno == EINTR) {
-            return TRUE;
-        } else if (errno == EAGAIN) {
-            return TRUE;
+    while (((result = nanosleep(&ts, &tsRemaining)) == -1) && (errno == EINTR)) {
+        if (interrupt) {
+            return (tsRemaining.tv_sec * 1000) + (tsRemaining.tv_nsec / 1000000);
         }
+        ts.tv_sec = tsRemaining.tv_sec;
+        ts.tv_nsec = tsRemaining.tv_nsec;
     }
+ #endif
 #endif
-#endif
-    return FALSE;
+    return result;
+}
+
+void wrapperSleep(int ms) {
+    wrapperSleepInterrupt(ms, FALSE);
 }
 
 /**
@@ -208,28 +225,17 @@ jstring JNU_NewStringFromNativeMB(JNIEnv *env, const char *str) {
 jstring JNU_NewStringFromNativeW(JNIEnv *env, const TCHAR *strW) {
     jstring result;
     size_t len;
+#ifndef WIN32
     char* msgMB;
-#ifdef WIN32
-    int size;
-#else
     TCHAR* errorW;
 #endif
 
     len = _tcslen(strW);
-    if (len > 0) {
 #ifdef WIN32
-        size = WideCharToMultiByte(CP_UTF8, 0, strW, -1, NULL, 0, NULL, NULL);
-        if (size <= 0) {
-            _tprintf(TEXT("WrapperJNI Warn: Failed to convert string \"%s\": %s\n"), strW, getLastErrorText()); fflush(NULL);
-            return NULL;
-        }
-        msgMB = malloc(sizeof(char) * size);
-        if (!msgMB) {
-            throwOutOfMemoryError(env, TEXT("JNSN1"));
-            return NULL;
-        }
-        WideCharToMultiByte(CP_UTF8, 0, strW, -1, msgMB, size, NULL, NULL);
+    /* On Windows wchar_t is already UTF-16, like jchar, so we can just cast wchar_t* to jchar* and pass the string to NewString(), without conversion. */
+    result = (*env)->NewString(env, (const jchar *)strW, (jsize)len);
 #else
+    if (len > 0) {
         if (converterWideToMB(strW, &msgMB, MB_UTF8) < 0) {
             if (msgMB) {
                 /* An error message is stored in msgMB (we need to convert it to wide chars to display it). */
@@ -247,7 +253,6 @@ jstring JNU_NewStringFromNativeW(JNIEnv *env, const TCHAR *strW) {
             }
             return NULL;
         }
-#endif
     } else {
         /* We need to special case empty strings as some of the functions don't work correctly for them. */
         msgMB = malloc(sizeof(char) * 1);
@@ -259,6 +264,7 @@ jstring JNU_NewStringFromNativeW(JNIEnv *env, const TCHAR *strW) {
     }
     result = (*env)->NewStringUTF(env, msgMB);
     free(msgMB);
+#endif
     return result;
 }
 
@@ -379,9 +385,14 @@ void initUTF8Strings(JNIEnv *env) {
     utf8MethodGetProperty = getUTF8Chars(env, "getProperty");
     utf8SigLjavaLangStringrLjavaLangString = getUTF8Chars(env, "(Ljava/lang/String;)Ljava/lang/String;");
     utf8javaIOIOException = getUTF8Chars(env, "java/io/IOException");
+    utf8SigZ = getUTF8Chars(env, "Z");
     
 #ifdef WIN32
 #else
+    utf8javaioFileDescriptor = getUTF8Chars(env, "java/io/FileDescriptor");
+    utf8fd = getUTF8Chars(env, "fd");
+    utf8SigI = getUTF8Chars(env, "I");
+    utf8VrV = getUTF8Chars(env, "()V");
     utf8ClassOrgTanukisoftwareWrapperWrapperUNIXUser = getUTF8Chars(env, "org/tanukisoftware/wrapper/WrapperUNIXUser");
     utf8MethodSetGroup = getUTF8Chars(env, "setGroup");
     utf8MethodAddGroup = getUTF8Chars(env, "addGroup");
@@ -398,16 +409,18 @@ void initUTF8Strings(JNIEnv *env) {
  * @param env Current JNIEnv.
  * @param propertyName Name of the property.
  * @param propertyValue Value of the property, or NULL if it was not set.
+ * @param encodeNative The value of the property will actually a multibyte string (can be cast to char*)
  *
  * @return TRUE if there were any problems, FALSE if successful.  NULL values will still be successful.
  */
 int getSystemProperty(JNIEnv *env, const TCHAR *propertyName, TCHAR **propertyValue, int encodeNative) {
-    int result;
+    int result = FALSE;
     jclass jClassSystem;
     jmethodID jMethodIdGetProperty;
     jstring jStringKeyPropName;
     jstring jStringKeyValue;
-    TCHAR *keyChars;
+    const char *valueMB;
+    size_t len;
 
     /* Initialize the propertyValue to point to NULL in case we fail. */
     *propertyValue = NULL;
@@ -418,33 +431,23 @@ int getSystemProperty(JNIEnv *env, const TCHAR *propertyName, TCHAR **propertyVa
                 if ((jStringKeyValue = (jstring)(*env)->CallStaticObjectMethod(env, jClassSystem, jMethodIdGetProperty, jStringKeyPropName)) != NULL) {
                     /* Collect the value. */
                     if (!encodeNative) {
-                        if ((keyChars = JNU_GetNativeWFromString(env, jStringKeyValue)) != NULL) {
-                            *propertyValue = malloc(sizeof(TCHAR) * (_tcslen(keyChars) + 1));
-                            if (!*propertyValue) {
-                                throwOutOfMemoryError(env, TEXT("GSP1"));
-                                result = TRUE;
-                            } else {
-                                _tcsncpy(*propertyValue, keyChars, _tcslen(keyChars) + 1);
-                                result = FALSE;
-                            }
-                            
-                            free(keyChars);
-                        } else {
+                        *propertyValue = JNU_GetNativeWFromString(env, jStringKeyValue);
+                        if (*propertyValue == NULL) {
                             /* Exception Thrown */
                             result = TRUE;
                         }
                     } else {
-                        if ((keyChars = (TCHAR*)(*env)->GetStringUTFChars(env, jStringKeyValue, NULL)) != NULL) {
-                            *propertyValue = malloc(strlen((char*)keyChars) + 1);
-                            if (!*propertyValue) {
-                                throwOutOfMemoryError(env, TEXT("GSP2"));
+                        valueMB = (*env)->GetStringUTFChars(env, jStringKeyValue, NULL);
+                        if (valueMB) {
+                            len = strlen(valueMB);
+                            *propertyValue = malloc(len + 1);
+                            if (*propertyValue == NULL) {
+                                throwOutOfMemoryError(env, TEXT("GSP1"));
                                 result = TRUE;
                             } else {
-                                strncpy((char*)*propertyValue, (char*)keyChars, strlen((char*)keyChars) + 1);
-                                result = FALSE;
+                                strncpy((char*)(*propertyValue), valueMB, len + 1);
                             }
-                            
-                            (*env)->ReleaseStringUTFChars(env, jStringKeyValue, (const char *)keyChars);
+                            (*env)->ReleaseStringUTFChars(env, jStringKeyValue, valueMB);
                         } else {
                             /* Exception Thrown */
                             result = TRUE;
@@ -477,10 +480,46 @@ static JavaVM *jvm = NULL;
 static jobject outGlobalRef = NULL;
 static jmethodID printlnMethodId = NULL;
 
-int printMessageCallback(const TCHAR* message) {
+static JNIEnv* getEnv(int* pAttached) {
     JNIEnv** envPtr;
     JNIEnv* env = NULL;
+    jint ret1;
+    jint ret2;
+    int attached = FALSE;
+
+    /* envPtr is to avoid a warning "dereferencing type-punned pointer will break strict-aliasing rules" happening on certain platforms. */
+    envPtr = &env;
+
+    if (jvm) { /* set when the libary is initialized */
+        ret1 = (*jvm)->GetEnv(jvm, (void **)envPtr, JNI_VERSION_1_4);
+        if (!env && ((pAttached != NULL) || (ret1 != JNI_EDETACHED))) {
+            /* The current thread was not attached (JNI_EDETACHED), or the jni version was not supported (JNI_EVERSION).
+             *  For JNI_EVERSION, AttachCurrentThread() will not affect the thread, but we can still use it to retrieve
+             *  the environment (see its source code). */
+            ret2 = (*jvm)->AttachCurrentThread(jvm, (void**)envPtr, NULL);
+            if ((ret1 == JNI_EDETACHED) && (ret2 == JNI_OK)) {
+                attached = TRUE;
+            }
+        }
+    }
+    if (pAttached) {
+        *pAttached = attached;
+    }
+    return env;
+}
+
+void detachCurrentThread() {
+    jint ret = (*jvm)->DetachCurrentThread(jvm);
+    if (ret != JNI_OK) {
+        _tprintf(TEXT("Failed to detach current thread - %d.\n"), ret); fflush(NULL);
+    }
+}
+
+int printMessageCallback(const TCHAR* message) {
+    int threadWasAttached = FALSE;
+    JNIEnv* env;
     jstring jMsg;
+    int result = TRUE;
 
     /* Do not print directly to the standard output because the JVM will interpret it with its own encoding (file.encoding)
      *  which may differ from the locale encoding (especially on Windows or when setting file.encoding from the java additionals).
@@ -490,31 +529,37 @@ int printMessageCallback(const TCHAR* message) {
      *  messages to MB using the JVM encoding (we should however use the equivalent code page or iconv syntax). To do this, we would
      *  need to either include wrapper_jvminfo.c to the native libary (but it would make it heavier), or pass the equivalent encoding
      *  in a system property (the conversion would be the opposite as when the Wrapper catches JVM outputs). */
-    if (jvm && outGlobalRef && printlnMethodId) {
-        /* envPtr is to avoid a warning "dereferencing type-punned pointer will break strict-aliasing rules" happening on certain platforms. */
-        envPtr = &env;
-        if ((*jvm)->AttachCurrentThread(jvm, (void**)envPtr, NULL) == 0) {
+    if (outGlobalRef && printlnMethodId) {
+        env = getEnv(&threadWasAttached);
+        if (env) {
             if ((jMsg = JNU_NewStringFromNativeW(env, message)) != NULL) {
                 (*env)->CallVoidMethod(env, outGlobalRef, printlnMethodId, jMsg);
-                return FALSE;
+                result = FALSE;
             }
         }
+        if (threadWasAttached) {
+            detachCurrentThread();
+        }
     }
-    return TRUE;
+    return result;
 }
 
 /*
- * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Class:     org_tanukisoftware_wrapper_WrapperNativeLibrary
  * Method:    nativeDispose
  * Signature: (Z)V
  */
 JNIEXPORT void JNICALL
-Java_org_tanukisoftware_wrapper_WrapperManager_nativeDispose(JNIEnv *env, jclass jClassWrapperManager, jboolean debugging) {
+Java_org_tanukisoftware_wrapper_WrapperNativeLibrary_nativeDispose(JNIEnv *env, jclass clazz, jboolean debugging) {
     if (wrapperJNIDebugging) {
         /* This is useful for making sure that this is the last JNI call. */
-        log_printf(TEXT("WrapperJNI Debug: Disposing WrapperManager native library."));
+        log_printf(TEXT("WrapperJNI Debug: Disposing Wrapper native library."));
     }
+
+    /* First clear the message callback, then delete the global reference to the System.out object so it is not used after disposal. */
+    setPrintMessageCallback(NULL);
     (*env)->DeleteGlobalRef(env, outGlobalRef);
+    outGlobalRef = NULL;
 }
 
 /**
@@ -558,17 +603,20 @@ int initLog(JNIEnv *env) {
 }
 
 /**
- * Do common initializaion.
+ * Do common initialization.
  *
  * @return TRUE if there were any problems.
  */
-int initCommon(JNIEnv *env, jclass jClassWrapperManager) {
+int initCommon(JNIEnv *env) {
     TCHAR* outfile;
     TCHAR* errfile;
     int outfd;
     int errfd;
     int mode;
     int options;
+    jclass jClassWrapperManager;
+    jfieldID jFieldId;
+    jboolean jBooleanTestDisabled;
 #ifdef HPUX
     TCHAR* fixIconvHpuxPropValue = NULL;
 #endif
@@ -582,11 +630,25 @@ int initCommon(JNIEnv *env, jclass jClassWrapperManager) {
 #endif
     initUTF8Strings(env);
 
+    /* Check if test methods should be enabled.
+     *  Retrieve the information from the field rather than passing it as an argument to nativeInit(),
+     *  since this could fail if the wrapper.jar version and the native library version don't match.
+     *  The version check will occur subsequently, but initialization should still be allowed. */
+    if ((jClassWrapperManager = (*env)->FindClass(env, "org/tanukisoftware/wrapper/WrapperManager")) != NULL) {
+        if ((jFieldId = (*env)->GetStaticFieldID(env, jClassWrapperManager, getUTF8Chars(env, "TESTS_DISABLED"), utf8SigZ)) != NULL) {
+            jBooleanTestDisabled = (*env)->GetStaticBooleanField(env, jClassWrapperManager, jFieldId);
+            if (!jBooleanTestDisabled) {
+                testsDisabled = FALSE;
+            }
+        }
+    }
+
 #ifdef HPUX
     if (!getSystemProperty(env, TEXT("wrapper.fix_iconv_hpux"), &fixIconvHpuxPropValue, FALSE)) {
         if (fixIconvHpuxPropValue && (strcmpIgnoreCase(fixIconvHpuxPropValue, TEXT("ALWAYS")) == 0)) {
             toggleIconvHpuxFix(TRUE);
         }
+        free(fixIconvHpuxPropValue);
     }
 #endif
 
@@ -602,6 +664,7 @@ int initCommon(JNIEnv *env, jclass jClassWrapperManager) {
         } else {
             redirectedStdErr = TRUE;
         }
+        free(errfile);
     }
     if (getSystemProperty(env, TEXT("wrapper.java.outfile"), &outfile, FALSE)) {
         /* Failed */
@@ -615,12 +678,13 @@ int initCommon(JNIEnv *env, jclass jClassWrapperManager) {
         } else {
             redirectedStdOut = TRUE;
         }
+        free(outfile);
     }
-    
+
     return FALSE;
 }
 
-void throwThrowable(JNIEnv *env, char *throwableClassName, const TCHAR *lpszFmt, ...) {
+void throwThrowable(JNIEnv *env, const char *throwableClassName, const TCHAR *lpszFmt, ...) {
     va_list vargs;
     int messageBufferSize = 0;
     TCHAR *messageBuffer = NULL;
@@ -630,25 +694,27 @@ void throwThrowable(JNIEnv *env, char *throwableClassName, const TCHAR *lpszFmt,
     jstring jMessageBuffer;
     jobject jThrowable;
 #if defined(UNICODE) && !defined(WIN32)
+    size_t len;
     TCHAR *msg = NULL;
     int i;
     int flag;
-#endif
 
-#if defined(UNICODE) && !defined(WIN32)
     if (wcsstr(lpszFmt, TEXT("%s")) != NULL) {
-        msg = malloc(sizeof(wchar_t) * (wcslen(lpszFmt) + 1));
+        len = wcslen(lpszFmt);
+        msg = malloc(sizeof(wchar_t) * (len + 1));
         if (msg) {
             /* Loop over the format and convert all '%s' patterns to %S' so the UNICODE displays correctly. */
-            if (wcslen(lpszFmt) > 0) {
-                for (i = 0; i < _tcslen(lpszFmt); i++){
-                    msg[i] = lpszFmt[i];
-                    if ((lpszFmt[i] == TEXT('%')) && (i  < _tcslen(lpszFmt)) && (lpszFmt[i + 1] == TEXT('s')) && ((i == 0) || (lpszFmt[i - 1] != TEXT('%')))){
-                        msg[i+1] = TEXT('S'); i++;
+            for (i = 0; i < len; i++) {
+                msg[i] = lpszFmt[i];
+                if (lpszFmt[i] == TEXT('%') && (i + 1 < len)) {
+                    if (lpszFmt[i + 1] == TEXT('%')) {
+                        msg[i + 1] = lpszFmt[i + 1]; i++;
+                    } else if (lpszFmt[i + 1] == TEXT('s')) {
+                        msg[i + 1] = TEXT('S'); i++;
                     }
                 }
             }
-            msg[wcslen(lpszFmt)] = TEXT('\0');
+            msg[len] = TEXT('\0');
         } else {
             throwOutOfMemoryError(env, TEXT("TT0"));
             return;
@@ -672,7 +738,7 @@ void throwThrowable(JNIEnv *env, char *throwableClassName, const TCHAR *lpszFmt,
             if (!messageBuffer) {
                 throwOutOfMemoryError(env, TEXT("TT1"));
 #if defined(UNICODE) && !defined(WIN32)
-                if (flag == TRUE) {
+                if (flag) {
                     free(msg);
                 }
 #endif
@@ -715,7 +781,7 @@ void throwThrowable(JNIEnv *env, char *throwableClassName, const TCHAR *lpszFmt,
             if (!messageBuffer) {
                 throwOutOfMemoryError(env, TEXT("TT2"));
 #if defined(UNICODE) && !defined(WIN32)
-                if (flag == TRUE) {
+                if (flag) {
                     free(msg);
                 }
 #endif
@@ -728,7 +794,7 @@ void throwThrowable(JNIEnv *env, char *throwableClassName, const TCHAR *lpszFmt,
     } while (count < 0);
 
 #if defined(UNICODE) && !defined(WIN32)
-    if (flag == TRUE) {
+    if (flag) {
         free(msg);
     }
 #endif
@@ -758,7 +824,7 @@ void throwThrowable(JNIEnv *env, char *throwableClassName, const TCHAR *lpszFmt,
  * @param locationCode The locationCode to help tell where the error happened.
  */
 void throwOutOfMemoryError(JNIEnv *env, const TCHAR* locationCode) {
-    throwThrowable(env, (char*)utf8ClassJavaLangOutOfMemoryError, TEXT("Out of memory (%s)"), locationCode);
+    throwThrowable(env, utf8ClassJavaLangOutOfMemoryError, TEXT("Out of memory (%s)"), locationCode);
 
     log_printf(TEXT("WrapperJNI Error: Out of memory (%s)"), locationCode);
 }
@@ -813,6 +879,7 @@ void wrapperJNIHandleSignal(int signal) {
 }
 
 
+/** Keep this function for compatibility with older versions of the library. **/
 /*
  * Class:     org_tanukisoftware_wrapper_WrapperManager
  * Method:    nativeGetLibraryVersion
@@ -825,23 +892,36 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetLibraryVersion(JNIEnv *e
     return version;
 }
 
+/** Function introduced in version 3.6.3. **/
 /*
- * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Class:     org_tanukisoftware_wrapper_WrapperNativeLibrary
+ * Method:    nativeGetLibraryVersion
+ * Signature: ()Ljava/lang/String;
+ */
+JNIEXPORT jstring JNICALL
+Java_org_tanukisoftware_wrapper_WrapperNativeLibrary_nativeGetLibraryVersion(JNIEnv *env, jclass clazz) {
+    jstring version;
+    version = JNU_NewStringFromNativeW(env, wrapperVersion);
+    return version;
+}
+
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperNativeLibrary
  * Method:    nativeIsProfessionalEdition
  * Signature: ()Z
  */
 JNIEXPORT jboolean JNICALL
-Java_org_tanukisoftware_wrapper_WrapperManager_nativeIsProfessionalEdition(JNIEnv *env, jclass clazz) {
+Java_org_tanukisoftware_wrapper_WrapperNativeLibrary_nativeIsProfessionalEdition(JNIEnv *env, jclass clazz) {
     return JNI_FALSE;
 }
 
 /*
- * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Class:     org_tanukisoftware_wrapper_WrapperNativeLibrary
  * Method:    nativeIsStandardEdition
  * Signature: ()Z
  */
 JNIEXPORT jboolean JNICALL
-Java_org_tanukisoftware_wrapper_WrapperManager_nativeIsStandardEdition(JNIEnv *env, jclass clazz) {
+Java_org_tanukisoftware_wrapper_WrapperNativeLibrary_nativeIsStandardEdition(JNIEnv *env, jclass clazz) {
     return JNI_FALSE;
 }
 
@@ -882,6 +962,13 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetControlEvent(JNIEnv *env
     return event;
 }
 
+/**
+ * Read only accessor to spread the static value to other translation units.
+ */
+int isTestsDisabled() {
+    return testsDisabled;
+}
+
 /*
  * Class:     org_tanukisoftware_wrapper_WrapperManager
  * Method:    accessViolationInner
@@ -891,6 +978,10 @@ JNIEXPORT void JNICALL
 Java_org_tanukisoftware_wrapper_WrapperManager_accessViolationInner(JNIEnv *env, jclass clazz) {
     TCHAR *ptr;
 
+    if (isTestsDisabled()) {
+        return;
+    }
+
     log_printf(TEXT("WrapperJNI Warn: Causing access violation..."));
     
     /* Cause access violation */
@@ -899,8 +990,6 @@ Java_org_tanukisoftware_wrapper_WrapperManager_accessViolationInner(JNIEnv *env,
 
 }
 
-
-
 JNIEXPORT jobject JNICALL
 Java_org_tanukisoftware_wrapper_WrapperManager_nativeExec(JNIEnv *env, jclass jWrapperManagerClass, jobjectArray jCmdArray, jstring jCmdLine, jobject jWrapperProcessConfig, jboolean spawnChDir) {
 
