@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2020 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -39,11 +39,16 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <fcntl.h>
+#ifdef LINUX
+ #include <dirent.h>
+ #include <arpa/inet.h>
+ #include <ctype.h>
+#endif
 #include <unistd.h>
 #include "loggerjni.h"
 #include "wrapperjni.h"
 
-static pid_t wrapperProcessId = -1;
+pid_t wrapperProcessId = -1;
 pthread_mutex_t controlEventQueueMutex = PTHREAD_MUTEX_INITIALIZER;
 
 
@@ -53,7 +58,7 @@ int wrapperLockControlEventQueue() {
      *  This could happen if a signal is encountered while locked. */
     while (pthread_mutex_trylock(&controlEventQueueMutex) == EBUSY) {
         if (count >= 3000) {
-            log_printf(TEXT("WrapperJNI Error: Timed out waiting for control event queue lock."));
+            log_printf(TEXT("WrapperJNI Error: Timed out waiting for internal lock (%s)."), TEXT("control event queue"));
             return -1;
         }
         wrapperSleep(10);
@@ -63,17 +68,18 @@ int wrapperLockControlEventQueue() {
     if (count > 0) {
         if (wrapperJNIDebugging) {
             /* This is useful for making sure that the JNI call is working. */
-            log_printf(TEXT("WrapperJNI Debug: wrapperLockControlEventQueue looped %d times before lock."), count);
+            log_printf(TEXT("WrapperJNI Debug: Looped %d times before lock (%s)."), count, TEXT("control event queue"));
         }
     }
     return 0;
 }
 
 int wrapperReleaseControlEventQueue() {
-    if (pthread_mutex_unlock(&controlEventQueueMutex)) {
-        log_printf(TEXT("WrapperJNI Error: Failed to unlock the event queue mutex."));
+    int ret = pthread_mutex_unlock(&controlEventQueueMutex);
+    if (ret != 0) {
+        log_printf(TEXT("WrapperJNI Error: Failed to release internal lock (%s, 0x%x)."), TEXT("control event queue"), ret);
     }
-    return 0;
+    return ret;
 }
 
 /**
@@ -126,19 +132,17 @@ void handleHangup(int sig_num) {
 }
 */
 
-/*
- * Class:     org_tanukisoftware_wrapper_WrapperManager
- * Method:    nativeInit
- * Signature: (Z)V
+/**
+ * Basic initialization when the library is loaded.
  */
-JNIEXPORT void JNICALL
-Java_org_tanukisoftware_wrapper_WrapperManager_nativeInit(JNIEnv *env, jclass jClassWrapperManager, jboolean debugging) {
+JNIEXPORT jint JNICALL
+JNI_OnLoad(JavaVM *vm, void *reserved) {
     TCHAR *retLocale;
-    wrapperJNIDebugging = debugging;
+
 #ifdef FREEBSD
     if (loadIconvLibrary()) {
         /* Already reported. */
-        return;
+        return JNI_ERR;
     }
 #endif
 
@@ -149,11 +153,25 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeInit(JNIEnv *env, jclass jC
         free(retLocale);
     }
 #endif
+
+    /* Currently supporting Java 1.4+. */
+    return JNI_VERSION_1_4;
+}
+
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperNativeLibrary
+ * Method:    nativeInit
+ * Signature: (Z)V
+ */
+JNIEXPORT void JNICALL
+Java_org_tanukisoftware_wrapper_WrapperNativeLibrary_nativeInit(JNIEnv *env, jclass clazz, jboolean debugging) {
+    wrapperJNIDebugging = debugging;
+
     initLog(env);
 
     if (wrapperJNIDebugging) {
         /* This is useful for making sure that the JNI call is working. */
-        log_printf(TEXT("WrapperJNI Debug: Inside native WrapperManager initialization method"));
+        log_printf(TEXT("WrapperJNI Debug: Initializing Wrapper native library."));
     }
 
     /* Set handlers for signals */
@@ -165,7 +183,7 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeInit(JNIEnv *env, jclass jC
     signal(SIGUSR2, handleUsr2);
     */
 
-    if (initCommon(env, jClassWrapperManager)) {
+    if (initCommon(env)) {
         /* Failed.  An exception will have been thrown. */
         return;
     }
@@ -174,6 +192,15 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeInit(JNIEnv *env, jclass jC
     wrapperProcessId = getpid();
 }
 
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Method:    nativeLoadWrapperProperties
+ * Signature: ()V
+ */
+JNIEXPORT void JNICALL
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeLoadWrapperProperties(JNIEnv *env, jclass clazz) {
+}
+
 /*
  * Class:     org_tanukisoftware_wrapper_WrapperManager
  * Method:    nativeRaiseExceptionInner
@@ -181,6 +208,10 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeInit(JNIEnv *env, jclass jC
  */
 JNIEXPORT void JNICALL
 Java_org_tanukisoftware_wrapper_WrapperManager_nativeRaiseExceptionInner(JNIEnv *env, jclass clazz, jint code) {
+    if (isTestsDisabled()) {
+        return;
+    }
+
     log_printf(TEXT("WrapperJNI: Not Windows.  RaiseException ignored."));
 }
 
@@ -191,6 +222,10 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeRaiseExceptionInner(JNIEnv
  */
 JNIEXPORT void JNICALL
 Java_org_tanukisoftware_wrapper_WrapperManager_nativeRaiseFailFastExceptionInner(JNIEnv *env, jclass clazz) {
+    if (isTestsDisabled()) {
+        return;
+    }
+
     log_printf(TEXT("WrapperJNI: Not Windows.  FailFastException ignored."));
 }
 
@@ -200,7 +235,7 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeRaiseFailFastExceptionInner
  * Signature: ()I
  */
 JNIEXPORT jint JNICALL
-Java_org_tanukisoftware_wrapper_WrapperManager_nativeRedirectPipes(JNIEnv *evn, jclass clazz) {
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeRedirectPipes(JNIEnv *env, jclass clazz) {
     int fd;
     
     fd = _topen(TEXT("/dev/null"), O_RDWR, 0);
@@ -229,6 +264,38 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeRedirectPipes(JNIEnv *evn,
     return 0;
 }
 
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Method:    nativeConnectBackendPipeDescriptor
+ * Signature: (I)Ljava/io/FileDescriptor;
+ */
+JNIEXPORT jobject JNICALL
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetFileDescriptor(JNIEnv *env, jclass clazz, jint jintFd) {
+    jclass fdClass;
+    jfieldID fdFieldID;
+    jmethodID fdConstructor;
+    jobject fileDesc = NULL;
+
+    /* Note: If any of the steps below fail, an exception will be thrown, so the function should never return NULL. */
+
+    /* Get the FileDescriptor class. */
+    if ((fdClass = (*env)->FindClass(env, utf8javaioFileDescriptor)) != NULL) {
+        /* Get the private "fd" field. */
+        if ((fdFieldID = (*env)->GetFieldID(env, fdClass, utf8fd, utf8SigI)) != NULL) {
+            /* Get the constructor. */
+            if ((fdConstructor = (*env)->GetMethodID(env, fdClass, utf8MethodInit, utf8VrV)) != NULL) {
+                /* Create a FileDescriptor instance (this is done last, so we don't need to release the instance if other steps fail). */
+                if ((fileDesc = (*env)->NewObject(env, fdClass, fdConstructor)) != NULL) {
+                    /* Set "fd" with the file descriptor number (jintFd). */
+                    (*env)->SetIntField(env, fileDesc, fdFieldID, jintFd);
+                }
+            }
+        }
+        (*env)->DeleteLocalRef(env, fdClass);
+    }
+    return fileDesc;
+}
+
 /*
  * Class:     org_tanukisoftware_wrapper_WrapperManager
  * Method:    nativeGetJavaPID
@@ -457,7 +524,8 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeSendServiceControlCode(JNIE
  * Method:    nativeGetPortStatus
  * Signature: (ILjava/lang/String;I)I
  *
- * @param port Port is the port whose status is requested.
+ * @param port The port whose status is requested.
+ * @param jAddress IPv4 or IPv6 address.
  * @param protocol The protocol of the port, 0=tcpv4, 1=tcpv6
  *
  * @return The status, -1=error, 0=closed, >0=in use.
@@ -468,6 +536,249 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetPortStatus(JNIEnv *env,
     return 0;
 }
 
+#ifdef LINUX
+/* Convert a string IPv4 address to a 32-bit integer. */
+static unsigned long ipv4ToUlong(const char *ip_str) {
+    struct in_addr addr;
+    if (inet_pton(AF_INET, ip_str, &addr) != 1) {
+        return 0;
+    }
+    return addr.s_addr;
+}
+
+ #ifdef IPV6_SUPPORT
+/* Convert a string IPv6 address to a byte array. */
+static int fillIpv6ByteArray(unsigned char *buffer, const char *ip_str) {
+    struct in6_addr addr;
+    if (inet_pton(AF_INET6, ip_str, &addr) != 1) {
+        return -1;
+    }
+    memcpy(buffer, &addr, sizeof(addr));
+    return 0;
+}
+ #endif
+
+static int hasSocketDescriptor(pid_t pid, unsigned long inode) {
+    char buffer1[32];
+    char buffer2[256];
+    char *ptr = NULL;
+    size_t len1, len2;
+    DIR *fdDir;
+    struct dirent *entry;
+    unsigned long tempInode;
+    int result = FALSE;
+
+    snprintf(buffer1, sizeof(buffer1), "/proc/%d/fd", pid);
+    fdDir = opendir(buffer1);
+    if (fdDir) {
+        len1 = strlen(buffer1);
+        if (len1 < sizeof(buffer1) - 2) {
+            buffer1[len1] = '/';
+            while ((entry = readdir(fdDir)) != NULL) {
+                if (entry->d_type == DT_LNK) {
+                    snprintf(buffer1 + len1 + 1, sizeof(buffer1) - len1 - 1, "%.*s", (int)(sizeof(buffer1) - len1 - 1 - 1), entry->d_name);
+
+                    /* Read the symlink to get the info of the file descriptor. */
+                    len2 = readlink(buffer1, buffer2, sizeof(buffer2) - 1);
+                    if (len2 != -1) {
+                        buffer2[len2] = 0;
+                        if ((ptr = strstr(buffer2, "socket:[")) != NULL) {
+                            sscanf(ptr + 8, "%lx", &tempInode);
+                            if (tempInode == inode) {
+                                result = TRUE;
+                                break;
+                            }
+                        }
+                    }
+                }
+            }
+        }
+        closedir(fdDir);
+    }
+    return result;
+}
+#endif
+
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Method:    nativeGetSocketRemotePID
+ * Signature: (ILjava/lang/String;I)I
+ *
+ * @param port The remote port.
+ * @param jAddress IPv4 or IPv6 address.
+ * @param protocol The protocol of the port, 0=tcpv4, 1=tcpv6
+ * @param state The state, 0=listening, 1=established
+ *
+ * @return The pid of the remote process.
+ */
+JNIEXPORT jint JNICALL
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetSocketRemotePID(JNIEnv *env, jclass clazz, jint port, jstring jAddress, jint protocol, jint state) {
+    pid_t pid = 0;
+#ifdef LINUX
+    pid_t tempPid;
+    const char *nativeAddress;
+    unsigned long local_addr_ipv4 = 0;
+ #ifdef IPV6_SUPPORT
+    int ret;
+    unsigned char local_addr_ipv6[16] = {0};
+ #endif
+    int tcpState;
+    char buffer[256];
+    char *ptr1 = NULL;
+    char *ptr2 = NULL;
+    char *ptr3 = NULL;
+    TCHAR* fileName; 
+    FILE *file = NULL;
+    unsigned long local_addr;
+    unsigned int local_port;
+    unsigned int st = 0;
+    unsigned long inode = 0;
+    TCHAR* wrapperPidStr;
+    pid_t wrapperPid = 0;
+    DIR *procDir;
+    struct dirent *entry;
+
+    /* 1) Convert the address to the appropriate format (ipv4 uses ulong, ipv6 uses bytes array). */
+    nativeAddress = (*env)->GetStringUTFChars(env, jAddress, 0);
+    if (!nativeAddress) {
+        throwOutOfMemoryError(env, TEXT("GSRP2"));
+        return 0;
+    }
+    if (protocol == 0) {
+        /* IPv4 */
+        local_addr_ipv4 = ipv4ToUlong(nativeAddress);
+        (*env)->ReleaseStringUTFChars(env, jAddress, nativeAddress);
+        if (local_addr_ipv4 == 0) {
+            /* Failed. */
+            return 0;
+        }
+        fileName = TEXT("/proc/net/tcp");
+    } else {
+        /* IPv6 */
+ #ifdef IPV6_SUPPORT
+        ret = fillIpv6ByteArray(local_addr_ipv6, nativeAddress);
+        (*env)->ReleaseStringUTFChars(env, jAddress, nativeAddress);
+        if (ret == 0) {
+            /* Failed. */
+            return 0;
+        }
+        fileName = TEXT("/proc/net/tcp6");
+ #else
+        return 0;
+ #endif
+    }
+
+    /* 2) Figure out which state to select. */
+    switch (state) {
+    case 0:
+        tcpState = 10; /* TCP_LISTEN */
+        break;
+
+    case 1:
+        tcpState = 1; /* TCP_ESTABLISHED; */
+        break;
+
+    default:
+        /* Invalid. */
+        return 0;
+    }
+
+    /* 3) Seek for our socket in the tcp file and retrieve the inode. */
+    file = _tfopen(fileName, TEXT("r"));
+    if (!file) {
+        return 0;
+    }
+    if (fgets(buffer, sizeof(buffer), file)) {
+        /* First line is the header: '  sl  local_address rem_address   st tx_queue rx_queue tr tm->when retrnsmt   uid  timeout inode' */
+        ptr1 = strstr(buffer, "local_address");
+        ptr2 = strstr(buffer, "st ");
+        ptr3 = strstr(buffer, "inode");
+    }
+    if (ptr1 && ptr2 && ptr3) {
+        while (fgets(buffer, sizeof(buffer), file)) {
+ #ifdef IPV6_SUPPORT
+            /* Not implemented */
+            break;
+ #else
+            /* address:port */
+            if (sscanf(ptr1, "%lx:%x", &local_addr, &local_port) == 2) {
+                if ((local_addr == local_addr_ipv4) && (local_port == port)) {
+                    /* status (listening=0A, established=01) - IMPORTANT: there might be a delay for /proc/net/tcp to be updated! */
+                    if (sscanf(ptr2, "%x", &st) == 1) {
+                        if (st == tcpState) {
+                            /* inode */
+                            sscanf(ptr3, "%lx", &inode);
+                            break;
+                        }
+                    }
+                }
+            }
+ #endif
+        }
+    }
+    fclose(file);
+    if (inode == 0) {
+        return 0;
+    }
+
+    /* 4) Start by checking if the Wrapper owns the socket, as this is the most likely case. */
+    if (!getSystemProperty(env, TEXT("wrapper.pid"), &wrapperPidStr, FALSE) && wrapperPidStr) {
+        wrapperPid = _ttoi(wrapperPidStr);
+        if ((wrapperPid > 0) && hasSocketDescriptor(wrapperPid, inode)) {
+            return wrapperPid;
+        }
+    }
+
+    /* 5) Search for other processes: Iterate through the /proc folders and try to find the corresponding inode. */
+    procDir = opendir("/proc");
+    if (!procDir) {
+        return 0;
+    }
+    while ((entry = readdir(procDir)) != NULL) {
+        if ((entry->d_type == DT_DIR) && isdigit(entry->d_name[0])) {   /* look into directories whose name are PIDs */
+            tempPid = atoi(entry->d_name);
+            if ((tempPid != wrapperPid) && (tempPid != getpid())) {  /* make sure to not look into the folder of the current process */
+                if (hasSocketDescriptor(tempPid, inode)) {
+                    pid = tempPid;
+                    break;
+                }
+            }
+        }
+    }
+    closedir(procDir);
+#else
+    /* /proc can't be used on other Unix platforms.
+     * It is possible to retrieve the socket owner's PID using getsockopt() with SO_PEERCRED, but this only works with Unix domain sockets (AF_UNIX). */
+#endif
+    return pid;
+}
+
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Method:    nativeSetDpiAwareness
+ * Signature: (I)V
+ *
+ * @param awareness The DPI awareness value to set.
+ */
+JNIEXPORT void JNICALL
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeSetDpiAwareness(JNIEnv *env, jclass clazz, jint awareness) {
+    /* Not implemented on UNIX. */
+    return;
+}
+
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Method:    nativeGetDpiAwareness
+ * Signature: ()I
+ *
+ * @return The dpi Awareness of the current process, or -1 if there was an error.
+ */
+JNIEXPORT jint JNICALL
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetDpiAwareness(JNIEnv *env, jclass clazz) {
+    /* Not implemented on UNIX. */
+    return -1;
+}
+
 /*
  * Class:     org_tanukisoftware_wrapper_WrapperManager
  * Method:    nativeGetDpiScale
