@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2019 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -53,6 +53,9 @@ barf
 /* The largest possible command line length on Windows. */
 #define MAX_COMMAND_LINE_LEN 32766
 
+/* 20 days */
+#define WRAPPER_TIMEOUT_MAX      1728000
+
 /* MS Visual Studio 8 went and deprecated the POXIX names for functions.
  *  Fixing them all would be a big headache for UNIX versions. */
 #pragma warning(disable : 4996)
@@ -106,7 +109,7 @@ int wrapperReleaseControlEventQueue() {
         fflush(NULL);
     #endif
     if (!ReleaseMutex(controlEventQueueMutexHandle)) {
-        log_printf(TEXT( "WrapperJNI Error: Failed to release Control Event mutex. %s"), getLastErrorText());
+        log_printf(TEXT( "WrapperJNI Error: Failed to release internal lock (%s, 0x%x)."), TEXT("control event queue"), GetLastError());
         return -1;
     }
 
@@ -157,6 +160,11 @@ int wrapperConsoleHandler(int key) {
         log_printf(TEXT("WrapperJNI Debug: Handled signal"));
     }
 
+    if (key == CTRL_CLOSE_EVENT) {
+        /* Sleep enough time to let the JVM terminate cleanly (if we would let the handler complete, the process would exit). */
+        wrapperSleep(WRAPPER_TIMEOUT_MAX * 1000);
+    }
+
     return TRUE; /* We handled the event. */
 }
 
@@ -680,7 +688,7 @@ void commonExceptionInfo(PEXCEPTION_POINTERS pExceptionInfo) {
                 log_printf(TEXT("WrapperJNI Error:   DEP access exception to 0x%p"), pExceptionInfo->ExceptionRecord->ExceptionInformation[1]);
                 break;
             default:
-                log_printf(TEXT("WrapperJNI Error:   Unexpected(%d) access exception to 0x%p"), pExceptionInfo->ExceptionRecord->ExceptionInformation[0], pExceptionInfo->ExceptionRecord->ExceptionInformation[1]);
+                log_printf(TEXT("WrapperJNI Error:   Unexpected access exception to 0x%p (%d)"), pExceptionInfo->ExceptionRecord->ExceptionInformation[1], pExceptionInfo->ExceptionRecord->ExceptionInformation[0]);
                 break;
             }
         }
@@ -706,14 +714,26 @@ LONG WINAPI wrapperVectoredExceptionHandler(PEXCEPTION_POINTERS pExceptionInfo)
     
     return EXCEPTION_CONTINUE_SEARCH;
 }
-        
+
+/**
+ * Basic initialization when the library is loaded.
+ */
+JNIEXPORT jint JNICALL
+JNI_OnLoad(JavaVM *vm, void *reserved) {
+    /* Set the locale so we can display MultiByte characters. */
+    _tsetlocale(LC_ALL, TEXT(""));
+
+    /* Currently supporting Java 1.4+. */
+    return JNI_VERSION_1_4;
+}
+
 /*
- * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Class:     org_tanukisoftware_wrapper_WrapperNativeLibrary
  * Method:    nativeInit
  * Signature: (Z)V
  */
 JNIEXPORT void JNICALL
-Java_org_tanukisoftware_wrapper_WrapperManager_nativeInit(JNIEnv *env, jclass jClassWrapperManager, jboolean debugging) {
+Java_org_tanukisoftware_wrapper_WrapperNativeLibrary_nativeInit(JNIEnv *env, jclass clazz, jboolean debugging) {
     TCHAR szPath[_MAX_PATH];
     DWORD usedLen;
     OSVERSIONINFO osVer;
@@ -723,13 +743,11 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeInit(JNIEnv *env, jclass jC
     
     wrapperJNIDebugging = debugging;
 
-    /* Set the locale so we can display MultiByte characters. */
-    _tsetlocale(LC_ALL, TEXT(""));
     initLog(env);
 
     if (wrapperJNIDebugging) {
         /* This is useful for making sure that the JNI call is working. */
-        log_printf(TEXT("WrapperJNI Debug: Initializing WrapperManager native library."));
+        log_printf(TEXT("WrapperJNI Debug: Initializing Wrapper native library."));
 
         /* Important : For win XP getLastError() is unchanged if the buffer is too small, so if we don't reset the last error first, we may actually test an old pending error. */
         SetLastError(ERROR_SUCCESS);
@@ -764,7 +782,7 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeInit(JNIEnv *env, jclass jC
         log_printf(TEXT("WrapperJNI Error:  If the system shuts down while the application is still running, the Java process will be forcibly killed."));
     }
 
-    if (initCommon(env, jClassWrapperManager)) {
+    if (initCommon(env)) {
         /* Failed.  An exception will have been thrown. */
         return;
     }
@@ -817,6 +835,15 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeInit(JNIEnv *env, jclass jC
     initExplorerExeName();
 }
 
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Method:    nativeLoadWrapperProperties
+ * Signature: ()V
+ */
+JNIEXPORT void JNICALL
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeLoadWrapperProperties(JNIEnv *env, jclass clazz) {
+}
+
 /*
  * Class:     org_tanukisoftware_wrapper_WrapperManager
  * Method:    nativeRaiseExceptionInner
@@ -824,6 +851,10 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeInit(JNIEnv *env, jclass jC
  */
 JNIEXPORT void JNICALL
 Java_org_tanukisoftware_wrapper_WrapperManager_nativeRaiseExceptionInner(JNIEnv *env, jclass clazz, jint code) {
+    if (isTestsDisabled()) {
+        return;
+    }
+
     log_printf(TEXT("WrapperJNI Warn: Raising Exception 0x%08x..."), code);
     
     RaiseException(code, EXCEPTION_NONCONTINUABLE, 0, NULL);
@@ -836,6 +867,10 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeRaiseExceptionInner(JNIEnv
  */
 JNIEXPORT void JNICALL
 Java_org_tanukisoftware_wrapper_WrapperManager_nativeRaiseFailFastExceptionInner(JNIEnv *env, jclass clazz) {
+    if (isTestsDisabled()) {
+        return;
+    }
+
     if (OptionalRaiseFailFastException != NULL) {
         log_printf(TEXT("WrapperJNI Warn: Raising FailFastException..."));
         
@@ -851,7 +886,7 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeRaiseFailFastExceptionInner
  * Signature: ()I
  */
 JNIEXPORT jint JNICALL
-Java_org_tanukisoftware_wrapper_WrapperManager_nativeRedirectPipes(JNIEnv *evn, jclass clazz) {
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeRedirectPipes(JNIEnv *env, jclass clazz) {
     /* We don't need to do anything on Windows. */
     return 0;
 }
@@ -1513,7 +1548,8 @@ int getPortStatusTCPv4(JNIEnv *env, int port, jstring jAddress) {
  * Method:    nativeGetPortStatus
  * Signature: (ILjava/lang/String;I)I
  *
- * @param port Port is the port whose status is requested.
+ * @param port The port whose status is requested.
+ * @param jAddress IPv4 or IPv6 address.
  * @param protocol The protocol of the port, 0=tcpv4, 1=tcpv6
  *
  * @return The status, -1=error, 0=closed, >0=in use.
@@ -1532,6 +1568,190 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetPortStatus(JNIEnv *env,
     }
 }
 
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Method:    nativeGetSocketRemotePID
+ * Signature: (ILjava/lang/String;I)I
+ *
+ * @param port The remote port.
+ * @param jAddress IPv4 or IPv6 address.
+ * @param protocol The protocol of the port, 0=tcpv4, 1=tcpv6
+ * @param state The state, 0=listening, 1=established
+ *
+ * @return The pid of the remote process.
+ */
+JNIEXPORT jint JNICALL
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetSocketRemotePID(JNIEnv *env, jclass clazz, jint port, jstring jAddress, jint protocol, jint state) {
+    jint pid = 0;
+    DWORD dwSize = 0;
+    DWORD dwRetVal = 0;
+    DWORD i;
+    ULONG ulAf;
+    int tcpState;
+    const char *nativeAddress;
+    PVOID pTcpTable;
+    PMIB_TCPTABLE_OWNER_PID pTcp4Table;
+    struct in_addr addr;
+    char *localAddress4;
+#ifdef IPV6_SUPPORT
+    PMIB_TCP6TABLE_OWNER_PID pTcp6Table;
+    struct sockaddr_in6 sa;
+    char localAddress6[INET6_ADDRSTRLEN];
+#endif
+
+    if (protocol == 0) {
+        /* IPv4 */
+        ulAf = AF_INET;
+    } else {
+        /* IPv6 */
+#ifdef IPV6_SUPPORT
+        ulAf = AF_INET6;
+#else
+        return 0;
+#endif
+    }
+
+    switch (state) {
+    case 0:
+        tcpState = 2; /* MIB_TCP_STATE_LISTEN */
+        break;
+
+    case 1:
+        tcpState = 5; /* MIB_TCP_STATE_ESTAB; */
+        break;
+
+    default:
+        /* Invalid. */
+        return 0;
+    }
+
+    dwRetVal = GetExtendedTcpTable(NULL, &dwSize, FALSE, ulAf, TCP_TABLE_OWNER_PID_ALL, 0);
+    if (dwRetVal == ERROR_INSUFFICIENT_BUFFER) {
+        pTcpTable = malloc(dwSize);
+        if (!pTcpTable) {
+            throwOutOfMemoryError(env, TEXT("GSRP1"));
+        } else {
+            dwRetVal = GetExtendedTcpTable(pTcpTable, &dwSize, FALSE, ulAf, TCP_TABLE_OWNER_PID_ALL, 0);
+            if (dwRetVal == NO_ERROR) {
+                nativeAddress = (*env)->GetStringUTFChars(env, jAddress, 0);
+                if (!nativeAddress) {
+                    throwOutOfMemoryError(env, TEXT("GSRP2"));
+                } else {
+                    if (protocol == 0) {
+                        pTcp4Table = (PMIB_TCPTABLE_OWNER_PID)pTcpTable;
+                        for (i = 0; i < pTcp4Table->dwNumEntries; i++) {
+                            if (pTcp4Table->table[i].dwState == tcpState) {
+                                /* First check the port as this is faster.
+                                 *  note: GetExtendedTcpTable() only provide the PID of the local process that owns a TCP endpoint on the local machine (dwOwningPid).
+                                 *        To match the TCP entry with the PID of the remote process, we need to compare the remote port against dwLocalPort.
+                                 *        This works because both endpoint are the locale machine. */
+                                if (ntohs((u_short)pTcp4Table->table[i].dwLocalPort) == (int)port) {
+                                    /* Now check the address.
+                                     *  notes: - localAddress4 points to static buffer managed internally by the Ws2_32 library
+                                     *         - inet_ntoa does not require that the Windows Sockets DLL be loaded with WSAStartup() */
+                                    addr.s_addr = pTcp4Table->table[i].dwLocalAddr;
+                                    localAddress4 = inet_ntoa(addr);
+                                    if (strcmp(localAddress4, nativeAddress) == 0) {
+                                        pid = (jint)pTcp4Table->table[i].dwOwningPid;
+                                        break;
+                                    }
+                                }
+                            }
+                        }
+#ifdef IPV6_SUPPORT
+                    } else {
+                        pTcp6Table = (PMIB_TCP6TABLE_OWNER_PID)pTcpTable;
+                        for (i = 0; i < pTcp6Table->dwNumEntries; i++) {
+                            if (pTcp6Table->table[i].dwState == tcpState) {
+                                /* First check the port as this is faster.
+                                 *  note: GetExtendedTcpTable() only provide the PID of the local process that owns a TCP endpoint on the local machine (dwOwningPid).
+                                 *        To match the TCP entry with the PID of the remote process, we need to compare the remote port against dwLocalPort.
+                                 *        This works because both endpoint are the locale machine. */
+                                if (ntohs((u_short)pTcp6Table->table[i].dwLocalPort) == (int)port) {
+                                    /* Now check the address.
+                                     *  note: inet_ntop does not require that the Windows Sockets DLL be loaded with WSAStartup() */
+                                    sa.sin6_family = AF_INET6;
+                                    memcpy(&sa.sin6_addr, pTcp6Table->table[i].dwLocalAddr, sizeof(sa.sin6_addr));
+                                    inet_ntop(AF_INET6, &sa.sin6_addr, localAddress6, sizeof(localAddress6));
+                                    if (strcmp(localAddress6, nativeAddress) == 0) {
+                                        pid = (jint)pTcp6Table->table[i].dwOwningPid;
+                                        break;
+                                    }
+                                }
+                            }
+                        }
+#endif
+                    }
+                    (*env)->ReleaseStringUTFChars(env, jAddress, nativeAddress);
+                }
+            }
+            free(pTcpTable);
+        }
+    }
+    return pid;
+}
+
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Method:    nativeSetDpiAwareness
+ * Signature: (I)V
+ *
+ * @param awareness The DPI awareness value to set (0 = not aware, 1 = system DPI aware, 2 = per monitor DPI aware).
+ */
+JNIEXPORT void JNICALL
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeSetDpiAwareness(JNIEnv *env, jclass clazz, jint awareness) {
+    HMODULE scalingAPI;
+    FARPROC DynSetProcessDpiAwareness = NULL;
+    HRESULT ret;
+    
+    /* The ShellScalingAPI was added in windows 8.1 */
+    if ((scalingAPI = LoadLibrary(TEXT("Shcore.dll"))) == NULL) {
+        return;
+    } else if ((DynSetProcessDpiAwareness = GetProcAddress(scalingAPI, "SetProcessDpiAwareness")) == NULL) {
+        return;
+    }
+    
+    ret = (HRESULT)DynSetProcessDpiAwareness(awareness);
+    if (ret == E_INVALIDARG) {
+        throwThrowable(env, "java/lang/IllegalArgumentException", TEXT(""));
+    } else if (ret == E_ACCESSDENIED) {
+        throwThrowable(env, "java/io/IOException", TEXT("The DPI awareness is already set."));
+    }
+}
+
+/*
+ * Class:     org_tanukisoftware_wrapper_WrapperManager
+ * Method:    nativeGetDpiAwareness
+ * Signature: ()I
+ *
+ * @return The dpi Awareness of the current process (0 = not aware, 1 = system DPI aware, 2 = per monitor DPI aware), or -1 if there was an error.
+ */
+JNIEXPORT jint JNICALL
+Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetDpiAwareness(JNIEnv *env, jclass clazz) {
+    HMODULE scalingAPI;
+    FARPROC DynGetProcessDpiAwareness = NULL;
+    HRESULT ret;
+    int awareness; /* PROCESS_DPI_AWARENESS */
+    
+    /* The ShellScalingAPI was added in windows 8.1 */
+    if ((scalingAPI = LoadLibrary(TEXT("Shcore.dll"))) == NULL) {
+        return -1;
+    } else if ((DynGetProcessDpiAwareness = GetProcAddress(scalingAPI, "GetProcessDpiAwareness")) == NULL) {
+        return -1;
+    }
+
+    ret = (HRESULT)DynGetProcessDpiAwareness(NULL, &awareness);
+    if (ret == E_INVALIDARG) {
+        /* should not happen. */
+        throwThrowable(env, "java/lang/IllegalArgumentException", TEXT(""));
+        return -1;
+    } else if (ret == E_ACCESSDENIED) {
+        throwThrowable(env, "java/io/IOException", TEXT("The application does not have sufficient privileges."));
+        return -1;
+    }
+    return (jint)awareness;
+}
+
 /*
  * Class:     org_tanukisoftware_wrapper_WrapperManager
  * Method:    nativeGetDpiScale
@@ -1548,7 +1768,6 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetDpiScale(JNIEnv *env, jc
     int awareness; /* PROCESS_DPI_AWARENESS */
     HMODULE scalingAPI;
     FARPROC DynGetProcessDpiAwareness = NULL;
-    FARPROC DynSetProcessDpiAwareness = NULL;
     FARPROC DynGetDpiForMonitor = NULL;
     jint result;
     
@@ -1556,7 +1775,6 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetDpiScale(JNIEnv *env, jc
     if ((scalingAPI = LoadLibrary(TEXT("Shcore.dll"))) == NULL) {
         return 96; /* no scaling */
     } else if ((DynGetProcessDpiAwareness = GetProcAddress(scalingAPI, "GetProcessDpiAwareness")) == NULL ||
-               (DynSetProcessDpiAwareness = GetProcAddress(scalingAPI, "SetProcessDpiAwareness")) == NULL ||
                (DynGetDpiForMonitor = GetProcAddress(scalingAPI, "GetDpiForMonitor")) == NULL) {
         return 96; /* no scaling */
     }
@@ -1567,12 +1785,7 @@ Java_org_tanukisoftware_wrapper_WrapperManager_nativeGetDpiScale(JNIEnv *env, jc
     /* if awareness is PROCESS_SYSTEM_DPI_AWARE or PROCESS_PER_MONITOR_DPI_AWARE, 
      *  we can get the scale factor, else it will return 96 anyway so stop here. */
     if (awareness == 0) {
-        /* try to set the dpi awarness to PROCESS_PER_MONITOR_DPI_AWARE.
-         *  (this may be moved before the function call if we create a 
-         *  configuration property to control dpi awareness.) */
-        if (DynSetProcessDpiAwareness(2) != S_OK) {
-            return 96;
-        }
+        return 96;
     }
     
     /* get the primary monitor */
