@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper;
 
 /*
- * Copyright (c) 1999, 2018 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -143,7 +143,7 @@ public class WrapperActionServer
                 catch ( UnsupportedEncodingException e )
                 {
                     /* This should not happen because we always make sure the encoding exists before launching a JVM. */
-                    System.out.println( WrapperManager.getRes().getString( "Failed to set the encoding '{0}' when creating a WrapperPrintStream.\n Make sure the value of sun.stdout.encoding is correct.", sunStdoutEncoding ) );
+                    System.out.println( WrapperManager.getRes().getString( "Failed to set the encoding ''{0}'' when creating a WrapperPrintStream.\n Make sure the value of sun.stdout.encoding is correct.", sunStdoutEncoding ) );
                 }
             }
         }
