@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper;
 
 /*
- * Copyright (c) 1999, 2020 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -40,10 +40,10 @@ import java.text.ParseException;
  *
  * @author Leif Mortenson <leif@tanukisoftware.com>
  */
-final class WrapperInfo
+public final class WrapperInfo
 {
     /** Version of the Wrapper. */
-    private static final String   m_version         = "3.5.43";
+    private static final String   m_version         = "3.6.3";
 
     /** Date that the Wrapper was built. */
     private static final Calendar m_build           = Calendar.getInstance();
@@ -56,8 +56,8 @@ final class WrapperInfo
         Calendar buildTime = Calendar.getInstance();
         try
         {
-            buildDate.setTime( new SimpleDateFormat( "yyyyMMdd" ).parse( "20200305" ) );
-            buildTime.setTime( new SimpleDateFormat( "HHmm" ).parse( "1255" ) );
+            buildDate.setTime( new SimpleDateFormat( "yyyyMMdd" ).parse( "20250909" ) );
+            buildTime.setTime( new SimpleDateFormat( "HHmm" ).parse( "1947" ) );
             
             m_build.set( buildDate.get( Calendar.YEAR ), 
                         buildDate.get( Calendar.MONTH ), 
@@ -77,7 +77,7 @@ final class WrapperInfo
      *
      * @return the version of the Wrapper.
      */
-    static String getVersion()
+    public static String getVersion()
     {
         return m_version;
     }
