@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper;
 
 /*
- * Copyright (c) 1999, 2020 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -30,6 +30,7 @@ public class WrapperProcess
 
     /* The PID of the process. */
     private int m_pid;
+    private int m_pgid;
     private int m_exitcode;
     private boolean m_isDetached;
     private int m_softShutdownTimeout;
@@ -85,6 +86,16 @@ public class WrapperProcess
         return m_pid;
     }
 
+    /**
+     * Returns the PGID of the process.
+     *
+     * @return The PGID of the process.
+     */
+    public int getPGID()
+    {
+        return m_pgid;
+    }
+
     /**
      * Gets the input stream of the subprocess. The stream obtains data piped
      *  from the standard output stream of the process represented by this
