@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper;
 
 /*
- * Copyright (c) 1999, 2020 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -62,7 +62,7 @@ public class WrapperProcessOutputStream
     {
         synchronized( this )
         {
-            if ( ( !m_closed ) && WrapperManager.isNativeLibraryOk() )
+            if ( ( !m_closed ) && WrapperNativeLibrary.allowsNativeCalls() )
             {
                 nativeWrite( b );
             }
@@ -87,7 +87,7 @@ public class WrapperProcessOutputStream
         {
             if ( !m_closed )
             {
-                if ( WrapperManager.isNativeLibraryOk() )
+                if ( WrapperNativeLibrary.allowsNativeCalls() )
                 {
                     nativeClose();
                 }
