@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper;
 
 /*
- * Copyright (c) 1999, 2020 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -69,7 +69,7 @@ public final class WrapperResources
                 catch ( UnsupportedEncodingException e )
                 {
                     /* This should not happen because we always make sure the encoding exists before launching a JVM. */
-                    System.out.println( WrapperManager.getRes().getString( "Failed to set the encoding '{0}' when creating a WrapperPrintStream.\n Make sure the value of sun.stdout.encoding is correct.", sunStdoutEncoding ) );
+                    System.out.println( WrapperNativeLibrary.getRes().getString( "Failed to set the encoding ''{0}'' when creating a WrapperPrintStream.\n Make sure the value of sun.stdout.encoding is correct.", sunStdoutEncoding ) );
                 }
             }
         }
@@ -98,15 +98,9 @@ public final class WrapperResources
     {
         try
         {
-            if ( WrapperManager.isLoggingFinalizers() )
-            {
-                // This can't be localized because of when it happens.
-                System.out.println( "WrapperResources.finalize Id=" + m_Id );
-            }
-            
             if ( m_Id != 0 )
             {
-                if ( WrapperManager.isNativeLibraryOk() )
+                if ( WrapperNativeLibrary.allowsNativeCalls() )
                 {
                     // clean up after the resource.
                     nativeDestroyResource();
@@ -153,11 +147,11 @@ public final class WrapperResources
                 //  If one of the following keys or their localized strings have such a problem this will recurse.
                 if ( localized )
                 {
-                    m_outError.println( WrapperManager.getRes().getString( "Localized resource string''s single quotes not escaped correctly: {0}", str ) );
+                    m_outError.println( WrapperNativeLibrary.getRes().getString( "Localized resource string''s single quotes not escaped correctly: {0}", str ) );
                 }
                 else
                 {
-                    m_outError.println( WrapperManager.getRes().getString( "Resource key''s single quotes not escaped correctly: {0}", str ) );
+                    m_outError.println( WrapperNativeLibrary.getRes().getString( "Resource key''s single quotes not escaped correctly: {0}", str ) );
                 }
                 break;
             }
@@ -183,7 +177,7 @@ public final class WrapperResources
             validateResourceKey( key, false );
         }
         
-        if ( ( m_Id != 0 ) && WrapperManager.isNativeLibraryOk() )
+        if ( ( m_Id != 0 ) && WrapperNativeLibrary.allowsNativeCalls() )
         {
             String str = nativeGetLocalizedString( key );
             if ( !str.equals( key ) )
