@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper;
 
 /*
- * Copyright (c) 1999, 2018 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -86,13 +86,13 @@ public class WrapperSimpleApp
     implements WrapperListener, Runnable
 {
     /** Info level log channel */
-    private static WrapperPrintStream m_outInfo;
+    private static WrapperPrintStream c_outInfo;
     
     /** Error level log channel */
-    private static WrapperPrintStream m_outError;
+    private static WrapperPrintStream c_outError;
     
     /** Debug level log channel */
-    private static WrapperPrintStream m_outDebug;
+    private static WrapperPrintStream c_outDebug;
     
     /**
      * Application's main method
@@ -151,6 +151,40 @@ public class WrapperSimpleApp
      */
     private Throwable m_initException;
     
+    /*---------------------------------------------------------------
+     * Static Methods
+     *-------------------------------------------------------------*/
+    static
+    {
+        // Set up some log channels
+        boolean streamsSet = false;
+        if ( "true".equals( System.getProperty( "wrapper.use_sun_encoding" ) ) ) {
+            String sunStdoutEncoding = System.getProperty( "sun.stdout.encoding" );
+            if ( ( sunStdoutEncoding != null ) && !sunStdoutEncoding.equals( System.getProperty( "file.encoding" ) ) ) {
+                /* We need to create the stream using the same encoding as the one used for stdout, else this will lead to encoding issues. */
+                try
+                {
+                    c_outInfo = new WrapperPrintStream( System.out, false, sunStdoutEncoding, "WrapperSimpleApp: " );
+                    c_outError = new WrapperPrintStream( System.out, false, sunStdoutEncoding, "WrapperSimpleApp Error: " );
+                    c_outDebug = new WrapperPrintStream( System.out, false, sunStdoutEncoding, "WrapperSimpleApp Debug: " );
+                    streamsSet = true;
+                }
+                catch ( UnsupportedEncodingException e )
+                {
+                    /* This should not happen because we always make sure the encoding exists before launching a JVM.
+                     *  If any of the above streams failed, we want to fall back to streams that use the same encoding. */
+                    System.out.println( WrapperManager.getRes().getString( "Failed to set the encoding ''{0}'' when creating a WrapperPrintStream.\n Make sure the value of sun.stdout.encoding is correct.", sunStdoutEncoding ) );
+                }
+            }
+        }
+        if ( !streamsSet )
+        {
+            c_outInfo = new WrapperPrintStream( System.out, "WrapperSimpleApp: " );
+            c_outError = new WrapperPrintStream( System.out, "WrapperSimpleApp Error: " );
+            c_outDebug = new WrapperPrintStream( System.out, "WrapperSimpleApp Debug: " );
+        }
+    }
+    
     /*---------------------------------------------------------------
      * Constructors
      *-------------------------------------------------------------*/
@@ -161,39 +195,8 @@ public class WrapperSimpleApp
      */
     protected WrapperSimpleApp( String args[] )
     {
-        
-        // Initialize the WrapperManager class on startup by referencing it.
-        Class wmClass = WrapperManager.class;
         m_mainMethod = null;
         
-        // Set up some log channels
-        boolean streamsSet = false;
-        if ( "true".equals( System.getProperty( "wrapper.use_sun_encoding" ) ) ) {
-            String sunStdoutEncoding = System.getProperty( "sun.stdout.encoding" );
-            if ( ( sunStdoutEncoding != null ) && !sunStdoutEncoding.equals( System.getProperty( "file.encoding" ) ) ) {
-                /* We need to create the stream using the same encoding as the one used for stdout, else this will lead to encoding issues. */
-                try
-                {
-                    m_outInfo = new WrapperPrintStream( System.out, false, sunStdoutEncoding, "WrapperSimpleApp: " );
-                    m_outError = new WrapperPrintStream( System.out, false, sunStdoutEncoding, "WrapperSimpleApp Error: " );
-                    m_outDebug = new WrapperPrintStream( System.out, false, sunStdoutEncoding, "WrapperSimpleApp Debug: " );
-                    streamsSet = true;
-                }
-                catch ( UnsupportedEncodingException e )
-                {
-                    /* This should not happen because we always make sure the encoding exists before launching a JVM.
-                     *  If any of the above streams failed, we want to fall back to streams that use the same encoding. */
-                    System.out.println( WrapperManager.getRes().getString( "Failed to set the encoding '{0}' when creating a WrapperPrintStream.\n Make sure the value of sun.stdout.encoding is correct.", sunStdoutEncoding ) );
-                }
-            }
-        }
-        if ( !streamsSet )
-        {
-            m_outInfo = new WrapperPrintStream( System.out, "WrapperSimpleApp: " );
-            m_outError = new WrapperPrintStream( System.out, "WrapperSimpleApp Error: " );
-            m_outDebug = new WrapperPrintStream( System.out, "WrapperSimpleApp Debug: " );
-        }
-        
         // Do all of our initialization here so the modified array list which is passed
         //  to the WrapperListener.start method can remain unchanged.  Ideally we would
         //  want to handle this within the start method, but that would be an API change
@@ -310,14 +313,8 @@ public class WrapperSimpleApp
             System.arraycopy( args, 1, appArgs, 0, appArgs.length );
         }
         
-        // Start the application.  If the JVM was launched from the native
-        //  Wrapper then the application will wait for the native Wrapper to
-        //  call the application's start method.  Otherwise the start method
-        //  will be called immediately.
-        WrapperManager.start( this, appArgs );
-        
-        // This thread ends, the WrapperManager will start the application after the Wrapper has
-        //  been properly initialized by calling the start method above.
+        // Store the appArgs so they can be used to start the application.
+        m_appArgs = appArgs;
     }
     
     /*---------------------------------------------------------------
@@ -340,7 +337,7 @@ public class WrapperSimpleApp
         {
             if ( WrapperManager.isDebugEnabled() )
             {
-                m_outDebug.println( WrapperManager.getRes().getString( "invoking main method" ) );
+                c_outDebug.println( WrapperManager.getRes().getString( "invoking main method" ) );
             }
             try
             {
@@ -361,7 +358,7 @@ public class WrapperSimpleApp
             
             if ( WrapperManager.isDebugEnabled() )
             {
-                m_outDebug.println( WrapperManager.getRes().getString( "main method completed" ) );
+                c_outDebug.println( WrapperManager.getRes().getString( "main method completed" ) );
             }
             
             synchronized(this)
@@ -393,13 +390,13 @@ public class WrapperSimpleApp
         
         // If we get here, then an error was thrown.  If this happened quickly 
         // enough, the start method should be allowed to shut things down.
-        m_outInfo.println();
-        m_outError.println( WrapperManager.getRes().getString( "Encountered an error running main:" ) );
+        c_outInfo.println();
+        c_outError.println( WrapperManager.getRes().getString( "Encountered an error running main:" ) );
 
         // We should print a stack trace here, because in the case of an 
         // InvocationTargetException, the user needs to know what exception
         // their app threw.
-        t.printStackTrace( m_outError );
+        t.printStackTrace( c_outError );
 
         synchronized( this )
         {
@@ -454,11 +451,11 @@ public class WrapperSimpleApp
         {
             if ( m_initError != null )
             {
-                m_outError.println( m_initError );
+                c_outError.println( m_initError );
             }
             if ( m_initException != null )
             {
-                m_initException.printStackTrace( m_outError );
+                m_initException.printStackTrace( c_outError );
             }
             if ( m_initShowUsage )
             {
@@ -483,7 +480,7 @@ public class WrapperSimpleApp
             maxLoops = Integer.MAX_VALUE;
             if ( WrapperManager.isDebugEnabled() )
             {
-                m_outDebug.println( WrapperManager.getRes().getString(
+                c_outDebug.println( WrapperManager.getRes().getString(
                     "start(args) Will wait indefinitely for the main method to complete." ) );
             }
         }
@@ -492,7 +489,7 @@ public class WrapperSimpleApp
             maxLoops = maxStartMainWait; // 1s loops.
             if ( WrapperManager.isDebugEnabled() )
             {
-                m_outDebug.println( WrapperManager.getRes().getString(
+                c_outDebug.println( WrapperManager.getRes().getString(
                         "start(args) Will wait up to {0} seconds for the main method to complete.",
                         new Integer(maxLoops) ) );
             }
@@ -501,7 +498,9 @@ public class WrapperSimpleApp
         Thread mainThread = new Thread( this, "WrapperSimpleAppMain" );
         synchronized(this)
         {
+            // Override the appArgs, they are usually the same as what was set in the constructor.
             m_appArgs = args;
+            
             mainThread.start();
             
             // Make sure the rest of this thread does not fall behind the application.
@@ -554,7 +553,7 @@ public class WrapperSimpleApp
             //  main method.
             if ( WrapperManager.isDebugEnabled() )
             {
-                m_outDebug.println( WrapperManager.getRes().getString(
+                c_outDebug.println( WrapperManager.getRes().getString(
                         "start(args) end.  Main Completed={0}, exitCode={1}",
                         new Boolean( m_mainComplete ), m_mainExitCode ) );
             }
@@ -569,7 +568,7 @@ public class WrapperSimpleApp
     {
         if ( WrapperManager.isDebugEnabled() )
         {
-            m_outDebug.println( WrapperManager.getRes().getString(
+            c_outDebug.println( WrapperManager.getRes().getString(
                     "stop({0})", new Integer ( exitCode ) ) );
         }
         
@@ -592,13 +591,13 @@ public class WrapperSimpleApp
             && ( WrapperManager.isLaunchedAsService() || WrapperManager.isIgnoreUserLogoffs() ) )
         {
             // Ignore
-            m_outInfo.println( WrapperManager.getRes().getString( "User logged out.  Ignored." ) );
+            c_outInfo.println( WrapperManager.getRes().getString( "User logged out.  Ignored." ) );
         }
         else
         {
             if ( WrapperManager.isDebugEnabled() )
             {
-                m_outDebug.println( WrapperManager.getRes().getString(
+                c_outDebug.println( WrapperManager.getRes().getString(
                         "controlEvent({0}) Stopping", new Integer( event ) ) );
             }
             WrapperManager.stop( 0 );
@@ -609,6 +608,16 @@ public class WrapperSimpleApp
     /*---------------------------------------------------------------
      * Methods
      *-------------------------------------------------------------*/
+    /**
+     * Returns the application arguments meant for the main method of the Main application class.
+     *
+     * @return the Application arguments.
+     */
+    protected String[] getAppArgs()
+    {
+        return m_appArgs;
+    }
+    
     /**
      * Displays application usage
      */
@@ -619,7 +628,7 @@ public class WrapperSimpleApp
         System.out.println( WrapperManager.getRes().getString(
             "WrapperSimpleApp Usage:" ) );
         System.out.println( WrapperManager.getRes().getString(
-            "  java org.tanukisoftware.wrapper.WrapperSimpleApp {app_class{/app_method}} [app_arguments]" ) );
+            "  java org.tanukisoftware.wrapper.WrapperSimpleApp '{'app_class'{'/app_method'}}' [app_arguments]" ) );
         System.out.println();
         System.out.println( WrapperManager.getRes().getString(
             "Where:" ) );
@@ -645,7 +654,24 @@ public class WrapperSimpleApp
      */
     public static void main( String args[] )
     {
-        new WrapperSimpleApp( args );
+        // Check the backend mode to see where the main args should come from.  This will also initialize the WrapperManger class.
+        if ( WrapperManager.hasBackendAppParameters() )
+        {
+            args = WrapperManager.getBackendAppParameters();
+        }
+       
+        // Create the WrapperSimpleApp instance which parses parses the full
+        //  list of arguments into a main class and arguments for that class.
+        WrapperSimpleApp wrapperSimpleApp = new WrapperSimpleApp( args );
+        
+        // Start the application.  If the JVM was launched from the native
+        //  Wrapper then the application will wait for the native Wrapper to
+        //  call the application's start method.  Otherwise the start method
+        //  will be called immediately.
+        WrapperManager.start( wrapperSimpleApp, wrapperSimpleApp.getAppArgs() );
+        
+        // This thread returns and ends, the WrapperManager will start the application after the Wrapper has
+        //  been properly initialized by calling the start method above.
     }
 }
 
