@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.event;
 
 /*
- * Copyright (c) 1999, 2011 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -39,16 +39,16 @@ package org.tanukisoftware.wrapper.event;
  *   <dd>The user logged off (Windows).</dd>
  *   <dt>WRAPPER_CTRL_SHUTDOWN_EVENT</dt>
  *   <dd>The system is being shutdown (Windows).</dd>
- *   <dt>WRAPPER_CTRL_TERM_EVENT</td>
+ *   <dt>WRAPPER_CTRL_TERM_EVENT</dt>
  *   <dd>The kill TERM signal was received (UNIX).</dd>
- *   <dt>WRAPPER_CTRL_HUP_EVENT</td>
+ *   <dt>WRAPPER_CTRL_HUP_EVENT</dt>
  *   <dd>The kill HUP signal was received (UNIX).</dd>
  * </dl>
  *
- * @author Leif Mortenson <leif@tanukisoftware.com>
+ * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
  */
 public class WrapperControlEvent
-    extends WrapperEvent
+    extends WrapperConsumableEvent
 {
     /**
      * Serial Version UID.
@@ -61,9 +61,6 @@ public class WrapperControlEvent
     /** The name of the event. */
     private String m_controlEventName;
     
-    /** True if the event has been consumed. */
-    private boolean m_consumed;
-    
     /*---------------------------------------------------------------
      * Constructors
      *-------------------------------------------------------------*/
@@ -126,29 +123,6 @@ public class WrapperControlEvent
         return m_controlEventName;
     }
     
-    /**
-     * Mark the event as consumed.  This should be done if the event
-     *  has been handled.
-     * <p>
-     * On Windows, some events are sent both to the JVM and Wrapper processes.
-     *  Event if the CTRL-C event is ignored within the JVM, the Wrapper
-     *  process may still initiate a shutdown.
-     */
-    public void consume()
-    {
-        m_consumed = true;
-    }
-    
-    /**
-     * Returns true if the event has been consumed.
-     *
-     * @return True if the event has been consumed.
-     */
-    public boolean isConsumed()
-    {
-        return m_consumed;
-    }
-    
     /**
      * Returns a string representation of the event.
      *
