@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.security;
 
 /*
- * Copyright (c) 1999, 2020 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -84,8 +84,8 @@ public class WrapperEventPermission
     
     public static String EVENT_TYPE_SERVICE = "service";
     public static String EVENT_TYPE_CONTROL = "control";
+    public static String EVENT_TYPE_REMOTE_CONTROL = "remoteControl";
     public static String EVENT_TYPE_CORE = "core";
-    public static String EVENT_TYPE_REMOTE_CONTROL = "remote control";
     
     private static int MASK_SERVICE = 1;
     private static int MASK_CONTROL = 2;
@@ -176,6 +176,18 @@ public class WrapperEventPermission
             }
             sb.append( EVENT_TYPE_CONTROL );
         }
+        if ( ( m_eventTypeMask & MASK_REMOTE_CONTROL ) != 0 )
+        {
+            if ( first )
+            {
+                sb.append( ',' );
+            }
+            else
+            {
+                first = false;
+            }
+            sb.append( EVENT_TYPE_REMOTE_CONTROL );
+        }
         if ( ( m_eventTypeMask & MASK_CORE ) != 0 )
         {
             if ( first )
@@ -299,6 +311,10 @@ public class WrapperEventPermission
         {
             return MASK_CONTROL;
         }
+        else if ( eventTypes == EVENT_TYPE_REMOTE_CONTROL )
+        {
+            return MASK_REMOTE_CONTROL;
+        }
         else if ( eventTypes == EVENT_TYPE_CORE )
         {
             return MASK_CORE;
@@ -321,6 +337,10 @@ public class WrapperEventPermission
             {
                 mask |= MASK_CONTROL;
             }
+            else if ( eventType.equals( EVENT_TYPE_REMOTE_CONTROL ) )
+            {
+                mask |= MASK_REMOTE_CONTROL;
+            }
             else if ( eventType.equals( EVENT_TYPE_CORE ) )
             {
                 mask |= MASK_CORE;
