@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.test;
 
 /*
- * Copyright (c) 1999, 2019 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -34,8 +34,12 @@ import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.IOException;
 import java.io.UnsupportedEncodingException;
+import java.util.ArrayList;
+import java.util.Collections;
 import java.util.Enumeration;
 import java.util.Properties;
+import java.text.SimpleDateFormat;
+import java.util.Date;
 
 import org.tanukisoftware.wrapper.WrapperManager;
 import org.tanukisoftware.wrapper.WrapperProcess;
@@ -214,7 +218,7 @@ public abstract class AbstractActionApp
                     /* This should not happen because we always make sure the encoding exists before launching a JVM.
                      *  If any of the above streams failed, we want to fall back to streams that use the same encoding.
                      *  This message can be localized because we did not redirect the streams yet. */
-                    System.out.println( Main.getRes().getString( "Failed to set the encoding '{0}' when creating a DeadlockPrintStream.\n Make sure the value of sun.stdout.encoding is correct.", sunStdoutEncoding ) );
+                    System.out.println( Main.getRes().getString( "Failed to set the encoding ''{0}'' when creating a DeadlockPrintStream.\n Make sure the value of sun.stdout.encoding is correct.", sunStdoutEncoding ) );
                 }
             }
         }
@@ -234,6 +238,24 @@ public abstract class AbstractActionApp
         WrapperManager.addWrapperEventListener( this, m_eventMask );
     }
     
+    private void dumpProperties( Properties props )
+    {
+        ArrayList nameList = new ArrayList();
+        
+        for ( Enumeration en = props.propertyNames(); en.hasMoreElements(); )
+        {
+            nameList.add( en.nextElement() );
+        }
+        
+        Collections.sort( nameList );
+        
+        for ( int i = 0; i < nameList.size(); i++ ) {
+            String name = (String)nameList.get( i );
+            System.out.println( "  " + name + "=" + props.getProperty( name ) );
+        }
+        System.out.println();
+    }
+    
     protected boolean doAction( String action )
     {
         if ( action.equals( "stop0" ) )
@@ -294,9 +316,7 @@ public abstract class AbstractActionApp
         }
         else if ( action.equals( "access_violation" ) )
         {
-            // The bug we used to cause this is not in most modern VMs so this is not shown by default.
-            WrapperManager.accessViolation();
-            
+            System.out.println( Main.getRes().getString( "The action is deprecated and has no effect. Use ''{0}'' instead.", "access_violation_native" ) );
         }
         else if ( action.equals( "access_violation_native" ) )
         {
@@ -471,12 +491,14 @@ public abstract class AbstractActionApp
                             String line;
                             try
                             {
-                                do {
+                                do
+                                {
                                     System.out.println( Main.getRes().getString( "Input an action (''help'' for a list of actions):") );
                                     
                                     int count = 0;
                                     int maxTries = 10;
-                                    while( true ) {
+                                    while( true )
+                                    {
                                         /* On some old JVMs, we may get an "Interrupted system call" exception if a child process is writing some output while tying to read from here.
                                          *  It would most likely work if we try again though. */
                                         try
@@ -492,22 +514,32 @@ public abstract class AbstractActionApp
                                             }
                                         }
                                     }
-                                    if ((line != null) && (!line.equals(""))) {
-                                        System.out.println(Main.getRes().getString( "Read action: {0}", line ) );
-                                        if ( !doAction( line ) )
+                                    if ( line != null )
+                                    {
+                                        if ( !line.equals("") )
                                         {
-                                            if ( !line.equals( "help" ) )
+                                            System.out.println(Main.getRes().getString( "Read action: {0}", line ) );
+                                            if ( !doAction( line ) )
                                             {
-                                                System.out.println( Main.getRes().getString( "Unknown action: {0}", line ) );
+                                                if ( !line.equals( "help" ) )
+                                                {
+                                                    System.out.println( Main.getRes().getString( "Unknown action: {0}", line ) );
+                                                }
+                                                printActions();
                                             }
-                                            printActions();
+                                        }
+                                        else
+                                        {
+                                            System.out.println(Main.getRes().getString( "Read action: <EMPTY>" ) );
                                         }
                                     }
                                     else
                                     {
-                                        System.out.println(Main.getRes().getString( "Read action: <EMPTY>" ) );
+                                        // EOF
+                                        System.out.println(Main.getRes().getString( "Stdin was closed." ) );
+                                        break;
                                     }
-                                } while (true);
+                                } while ( true );
                             }
                             catch ( IOException e )
                             {
@@ -540,24 +572,12 @@ public abstract class AbstractActionApp
         else if ( action.equals( "properties" ) )
         {
             System.out.println( Main.getRes().getString( "Dump System Properties:" ) );
-            Properties props = System.getProperties();
-            for ( Enumeration en = props.propertyNames(); en.hasMoreElements(); )
-            {
-                String name = (String)en.nextElement();
-                System.out.println( "  " + name + "=" + props.getProperty( name ) );
-            }
-            System.out.println();
+            dumpProperties( System.getProperties() );
         }
         else if ( action.equals( "configuration" ) )
         {
             System.out.println( Main.getRes().getString( "Dump Wrapper Properties:" ) );
-            Properties props = WrapperManager.getProperties();
-            for ( Enumeration en = props.propertyNames(); en.hasMoreElements(); )
-            {
-                String name = (String)en.nextElement();
-                System.out.println( "  " + name + "=" + props.getProperty( name ) );
-            }
-            System.out.println();
+            dumpProperties( WrapperManager.getProperties() );
         }
         else if ( action.equals( "listener" ) )
         {
@@ -566,86 +586,121 @@ public abstract class AbstractActionApp
         }
         else if ( action.equals( "service_list" ) )
         {
-            /*
-            for ( int i = 0; i < 1000; i++ )
+            if ( System.getSecurityManager() == null )
             {
-                WrapperWin32Service[] services = WrapperManager.listServices();
-            }
-            */
-            WrapperWin32Service[] services = WrapperManager.listServices();
-            if ( services == null )
-            {
-                System.out.println( Main.getRes().getString( "Services not supported by current platform." ) );
+                System.out.println( Main.getRes().getString( "This operation requires a SecurityManager to be set.\n  Please visit https://wrapper.tanukisoftware.com/doc/english/security-model.html" ) );
             }
             else
             {
-                System.out.println( Main.getRes().getString( "Registered Services:" ) );
-                for ( int i = 0; i < services.length; i++ )
+                /*
+                for ( int i = 0; i < 1000; i++ )
                 {
-                    System.out.println( "  " + services[i] );
+                    WrapperWin32Service[] services = WrapperManager.listServices();
+                }
+                */
+                WrapperWin32Service[] services = WrapperManager.listServices();
+                if ( services == null )
+                {
+                    System.out.println( Main.getRes().getString( "Services not supported by current platform." ) );
+                }
+                else
+                {
+                    System.out.println( Main.getRes().getString( "Registered Services:" ) );
+                    for ( int i = 0; i < services.length; i++ )
+                    {
+                        System.out.println( "  " + services[i] );
+                    }
                 }
             }
         }
         else if ( action.equals( "service_interrogate" ) )
         {
-            try
+            if ( System.getSecurityManager() == null )
             {
-                /*
-                for ( int i = 0; i < 10000; i++ )
+                System.out.println( Main.getRes().getString( "This operation requires a SecurityManager to be set.\n  Please visit https://wrapper.tanukisoftware.com/doc/english/security-model.html" ) );
+            }
+            else
+            {
+                try
                 {
+                    /*
+                    for ( int i = 0; i < 10000; i++ )
+                    {
+                        WrapperWin32Service service = WrapperManager.sendServiceControlCode(
+                            m_serviceName, WrapperManager.SERVICE_CONTROL_CODE_INTERROGATE );
+                    }
+                    */
                     WrapperWin32Service service = WrapperManager.sendServiceControlCode(
                         m_serviceName, WrapperManager.SERVICE_CONTROL_CODE_INTERROGATE );
+                    System.out.println( Main.getRes().getString( "Service after interrogate: {0}", service ) );
+                }
+                catch ( WrapperServiceException e )
+                {
+                    e.printStackTrace();
                 }
-                */
-                WrapperWin32Service service = WrapperManager.sendServiceControlCode(
-                    m_serviceName, WrapperManager.SERVICE_CONTROL_CODE_INTERROGATE );
-                System.out.println( Main.getRes().getString( "Service after interrogate: {0}", service ) );
-            }
-            catch ( WrapperServiceException e )
-            {
-                e.printStackTrace();
             }
         }
         else if ( action.equals( "service_start" ) )
         {
-            try
+            if ( System.getSecurityManager() == null )
             {
-                WrapperWin32Service service = WrapperManager.sendServiceControlCode(
-                    m_serviceName, WrapperManager.SERVICE_CONTROL_CODE_START );
-                System.out.println( Main.getRes().getString( "Service after start: {0}", service ) );
+                System.out.println( Main.getRes().getString( "This operation requires a SecurityManager to be set.\n  Please visit https://wrapper.tanukisoftware.com/doc/english/security-model.html" ) );
             }
-            catch ( WrapperServiceException e )
+            else
             {
-                e.printStackTrace();
+                try
+                {
+                    WrapperWin32Service service = WrapperManager.sendServiceControlCode(
+                        m_serviceName, WrapperManager.SERVICE_CONTROL_CODE_START );
+                    System.out.println( Main.getRes().getString( "Service after start: {0}", service ) );
+                }
+                catch ( WrapperServiceException e )
+                {
+                    e.printStackTrace();
+                }
             }
         }
         else if ( action.equals( "service_stop" ) )
         {
-            try
+            if ( System.getSecurityManager() == null )
             {
-                WrapperWin32Service service = WrapperManager.sendServiceControlCode(
-                    m_serviceName, WrapperManager.SERVICE_CONTROL_CODE_STOP );
-                System.out.println( Main.getRes().getString( "Service after stop: {0}", service ) );
+                System.out.println( Main.getRes().getString( "This operation requires a SecurityManager to be set.\n  Please visit https://wrapper.tanukisoftware.com/doc/english/security-model.html" ) );
             }
-            catch ( WrapperServiceException e )
+            else
             {
-                e.printStackTrace();
+                try
+                {
+                    WrapperWin32Service service = WrapperManager.sendServiceControlCode(
+                        m_serviceName, WrapperManager.SERVICE_CONTROL_CODE_STOP );
+                    System.out.println( Main.getRes().getString( "Service after stop: {0}", service ) );
+                }
+                catch ( WrapperServiceException e )
+                {
+                    e.printStackTrace();
+                }
             }
         }
         else if ( action.equals( "service_user" ) )
         {
-            try
+            if ( System.getSecurityManager() == null )
             {
-                for ( int i = 128; i < 256; i+=10 )
-                {
-                    WrapperWin32Service service = WrapperManager.sendServiceControlCode(
-                        m_serviceName, i );
-                    System.out.println( Main.getRes().getString( "Service after user code {0} : {1}", new Integer( i ), service ) );
-                }
+                System.out.println( Main.getRes().getString( "This operation requires a SecurityManager to be set.\n  Please visit https://wrapper.tanukisoftware.com/doc/english/security-model.html" ) );
             }
-            catch ( WrapperServiceException e )
+            else
             {
-                e.printStackTrace();
+                try
+                {
+                    for ( int i = 128; i < 256; i+=10 )
+                    {
+                        WrapperWin32Service service = WrapperManager.sendServiceControlCode(
+                            m_serviceName, i );
+                        System.out.println( Main.getRes().getString( "Service after user code {0} : {1}", new Integer( i ), service ) );
+                    }
+                }
+                catch ( WrapperServiceException e )
+                {
+                    e.printStackTrace();
+                }
             }
         }
         else if ( action.equals( "console_title" ) )
@@ -677,6 +732,11 @@ public abstract class AbstractActionApp
         {
             System.out.println( Main.getRes().getString( "Standard Edition: " ) + WrapperManager.isStandardEdition() );
         }
+        else if ( action.equals( "current_time" ) )
+        {
+            SimpleDateFormat df = new SimpleDateFormat( "yyyy-MM-dd HH:mm:ss.SSS" );
+            System.out.println( df.format( new Date() ) + " - " + System.getProperty( "user.timezone" ) );
+        }
         else if ( action.startsWith( "exec " ) && ( action.length() > 5 )  )
         {
             String command = action.substring( 5 );
@@ -867,6 +927,7 @@ public abstract class AbstractActionApp
         System.err.println( Main.getRes().getString( "   gc                       : Perform a GC sweep." ) );
         System.err.println( Main.getRes().getString( "   is_professional          : Displays whether or not this is a Professional Edition Wrapper." ) );
         System.err.println( Main.getRes().getString( "   is_standard              : Displays whether or not this is at least a Standard Edition Wrapper." ) );
+        System.err.println( Main.getRes().getString( "   current_time             : Prints the current time and timezone." ) );
         if ( WrapperManager.isProfessionalEdition() )
         {
             System.err.println( Main.getRes().getString( "   exec <cmd>               : Executes a managed child process." ) );
