@@ -1,7 +1,7 @@
 package org.tanukisoftware.wrapper.test;
 
 /*
- * Copyright (c) 1999, 2020 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2025 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -74,7 +74,7 @@ import org.tanukisoftware.wrapper.event.WrapperEvent;
  * <p>
  * <b>NOTE</b> that in most cases you will want to use Method #1, using the
  *  WrapperSimpleApp helper class to integrate your application.  Please
- *  see the <a href="http://wrapper.tanukisoftware.com/doc/english/integrate.html">integration</a>
+ *  see the <a href="https://wrapper.tanukisoftware.com/doc/english/integrate.html">integration</a>
  *  section of the documentation for more details.
  *
  * @author Tanuki Software Development Team &lt;support@tanukisoftware.com&gt;
@@ -122,7 +122,19 @@ public class Main
             
             try
             {
-                dpiScaleFactor = (float)WrapperManager.nativeGetDpiScale() / 96;
+                // Newer versions of Java are dpi aware per monitor (2) and will automatically rescale the GUI. Lower dpi awarenesses must be scaled manually.
+                int awareness = WrapperManager.nativeGetDpiAwareness();
+                if ( awareness < 2 )
+                {
+                    // Dpi awareness can only be changed if not set (unaware).
+                    if ( awareness == 0 )
+                    {
+                        // Set the dpi awareness to "per monitor".
+                        WrapperManager.nativeSetDpiAwareness(2);
+                    }
+                    // Calculate the dpi scale factor.
+                    dpiScaleFactor = (float)WrapperManager.nativeGetDpiScale() / 96;
+                }
             }
             catch ( UnsatisfiedLinkError e )
             {
@@ -281,23 +293,26 @@ public class Main
             
             buildCommand( panel, gridBag, c, getRes().getString( "Update Event Listener" ), "listener", flagPanel2 );
             
-            buildCommand( panel, gridBag, c, getRes().getString( "Service List" ), "service_list", getRes().getString( "Displays a list of registered services on Windows." ) );
-            
-            m_serviceName = new TextField( "testwrapper" );
-            Panel servicePanel = new Panel();
-            servicePanel.setLayout( new BorderLayout() );
-            servicePanel.add( new Label( getRes().getString( "Interrogate Service.  Service name: " ) ), BorderLayout.WEST );
-            servicePanel.add( m_serviceName, BorderLayout.CENTER );
-            Panel servicePanel2 = new Panel();
-            servicePanel2.setLayout( new BorderLayout() );
-            servicePanel2.add( servicePanel, BorderLayout.WEST );
-            buildCommand( panel, gridBag, c, getRes().getString( "Service Interrogate" ), "service_interrogate", servicePanel2 );
-            
-            buildCommand( panel, gridBag, c, getRes().getString( "Service Start" ), "service_start", getRes().getString( "Starts the above service." ) );
-            
-            buildCommand( panel, gridBag, c, getRes().getString( "Service Stop" ), "service_stop", getRes().getString( "Stops the above service." ) );
-            
-            buildCommand( panel, gridBag, c, getRes().getString( "Service User Code" ), "service_user", getRes().getString( "Sends a series of user codes to the above service." ) );
+            if ( WrapperManager.isWindows() )
+            {
+                buildCommand( panel, gridBag, c, getRes().getString( "Service List" ), "service_list", getRes().getString( "Displays a list of registered services on Windows." ) );
+                
+                m_serviceName = new TextField( "testwrapper" );
+                Panel servicePanel = new Panel();
+                servicePanel.setLayout( new BorderLayout() );
+                servicePanel.add( new Label( getRes().getString( "Interrogate Service.  Service name: " ) ), BorderLayout.WEST );
+                servicePanel.add( m_serviceName, BorderLayout.CENTER );
+                Panel servicePanel2 = new Panel();
+                servicePanel2.setLayout( new BorderLayout() );
+                servicePanel2.add( servicePanel, BorderLayout.WEST );
+                buildCommand( panel, gridBag, c, getRes().getString( "Service Interrogate" ), "service_interrogate", servicePanel2 );
+                
+                buildCommand( panel, gridBag, c, getRes().getString( "Service Start" ), "service_start", getRes().getString( "Starts the above service." ) );
+                
+                buildCommand( panel, gridBag, c, getRes().getString( "Service Stop" ), "service_stop", getRes().getString( "Stops the above service." ) );
+                
+                buildCommand( panel, gridBag, c, getRes().getString( "Service User Code" ), "service_user", getRes().getString( "Sends a series of user codes to the above service." ) );
+            }
             
             m_consoleTitle = new TextField( getRes().getString( "Java Service Wrapper" ) );
             Panel titlePanel = new Panel();
@@ -327,6 +342,8 @@ public class Main
             
             buildCommand( panel, gridBag, c, getRes().getString( "Is Standard?" ), "is_standard", getRes().getString( "Prints true if this is a Standard Edition." ) );
             
+            buildCommand( panel, gridBag, c, getRes().getString( "Print time" ), "current_time", getRes().getString( "Prints the current time and timezone." ) );
+            
             addWindowListener( this );
             
             // By default listen to Service and RemoteControl events.
@@ -441,7 +458,10 @@ public class Main
             }
             m_suspendSeconds.setText( Integer.toString( suspendSeconds ) );
             setSuspendSeconds( suspendSeconds );
-            setServiceName( m_serviceName.getText() );
+            if ( WrapperManager.isWindows() )
+            {
+                setServiceName( m_serviceName.getText() );
+            }
             setConsoleTitle( m_consoleTitle.getText() );
             setChildParams( m_childCommand.getText(), m_childDetached.getState() );
             
@@ -618,6 +638,18 @@ public class Main
             {
                 m_frame = new MainFrame();
                 m_frame.setVisible( true );
+                if ( WrapperManager.isMacOSX() )
+                {
+                    // On MacOS, the frame will not always be displayed correctly due to an initial paint timing problem.  Repainting after a moment solves the problem.
+                    try
+                    {
+                        Thread.sleep( 250 );
+                    }
+                    catch ( InterruptedException e )
+                    {
+                    }
+                    m_frame.repaint();
+                }
             }
             catch ( java.lang.InternalError e )
             {
@@ -770,8 +802,8 @@ public class Main
         {
             if ( WrapperManager.isLaunchedAsService() || WrapperManager.isIgnoreUserLogoffs() )
             {
-            System.out.println( getRes().getString( "TestWrapper:   Ignoring logoff event" ) );
-            // Ignore
+                System.out.println( getRes().getString( "TestWrapper:   Ignoring logoff event" ) );
+                // Ignore
             }
             else if ( !ignoreControlEvents() )
             {
@@ -886,15 +918,19 @@ public class Main
      */
     public static void main( String[] args )
     {
-       System.out.println( getRes().getString( "TestWrapper: Initializing..." ));
+        System.out.println( getRes().getString( "TestWrapper: Initializing..." ));
         
-       // System.out.println(wrm.getString("test 0 = {0} String = {1} double = {2} object = {3}", new Object[]{ String.valueOf(99), "test",String.valueOf(34.33), "testob"}) );
+        if ( WrapperManager.hasBackendAppParameters() )
+        {
+            args = WrapperManager.getBackendAppParameters();
+        }
+        
+        // System.out.println(wrm.getString("test 0 = {0} String = {1} double = {2} object = {3}", new Object[]{ String.valueOf(99), "test",String.valueOf(34.33), "testob"}) );
         // Start the application.  If the JVM was launched from the native
         //  Wrapper then the application will wait for the native Wrapper to
         //  call the application's start method.  Otherwise the start method
         //  will be called immediately.
         WrapperManager.start( new Main(), args );
-        
     }
 }
 
